using System;

class Vector
{
    private double[] values;

    // default c'tor: construct 1-dimensional null vector
    public Vector () : this (1)
    {
    }

    // user defined c'tor: construct n-dimensional null vector
    public Vector (int dim)
    {
        this.values = new double[dim];
    }

    // user defined c'tor: construct a vector from an array
    public Vector (double[] values)
    {
        // deep copy strategy
        this.values = new double[values.Length];
        for (int i = 0; i < values.Length; i ++)
            this.values[i] = values[i];
    }

    // dimension property
    public int Dimension
    {
        get
        {
            return this.values.Length;
        }
    }

    // length property
    public double Length
    {
        get
        {
            double len = 0.0;
            for (int i = 0; i < this.values.Length; i ++)
                len += (this[i] * this[i]);

            return Math.Sqrt (len);
        }
    }

    // indexer
    public double this[int i]
    {
        get
        {
            return this.values[i];
        }

        set
        {
            this.values[i] = value;
        }
    }

    // operators
    public static Vector operator + (Vector v1, Vector v2)
    {
        Vector tmp = new Vector (v1.values.Length);

        for (int i = 0; i < tmp.values.Length; i ++)
            tmp.values[i] =  v1.values[i] + v2.values[i];

        return tmp;
    }

    // vector multiplication with a scalar
    public static Vector operator * (Vector v, int scalar)
    {
        Vector tmp = new Vector (v.values);

        for (int i = 0; i < tmp.values.Length; i ++)
            tmp.values[i] *= scalar;

        return tmp;
    }

    public static Vector operator * (int scalar, Vector v)
    {
        return v * scalar;
    }

    // vector multiplication: scalar product
    public static double operator * (Vector v1, Vector v2)
    {
        double result = 0.0;

        for (int i = 0; i < v1.values.Length; i ++)
            result += v1.values[i] * v2.values[i];

        return result;
    }

    // comparison
    public static bool operator == (Vector v1, Vector v2)
    {
        if (v1.values.Length != v2.values.Length)
            return false;

        for (int i = 0; i < v1.values.Length; i ++)
            if (v1.values[i] != v2.values[i])
                return false;

        return true;
    }

    public static bool operator != (Vector v1, Vector v2)
    {
        return ! (v1 == v2);
    }

    // simple output
    public void Print ()
    {
        Console.Write ("[");
        for (int i = 0; i < this.values.Length; i ++)
        {
            Console.Write ("{0,4}", this.values[i]);
            if (i < this.values.Length - 1)
                Console.Write (",");
        }
        Console.Write ("]");
    }
}

class Demo
{
    public static void Main ()
    {
        /** testing c'tors */
        Console.Write ("Default c'tor:              ");
        Vector v1 = new Vector ();
        v1.Print ();
        Console.WriteLine ();

        Console.Write ("User defined c'tor (int):   ");
        Vector v2 = new Vector (3);
        v2.Print ();
        Console.WriteLine ();

        Console.Write ("User defined c'tor (int[]): ");
        double[] values = { 1.0, 2.0, 3.0, 4.0, 5.0 };
        Vector v3 = new Vector (values);
        v3.Print ();
        Console.WriteLine ();

        /** testing properties and indexer */
        v3[1] = 17;
        Console.Write ("v3[1] = 17:                 ");
        v3.Print ();
        Console.WriteLine ();

        double val = v3[1];
        int dim = v3.Dimension;
        double len = v3.Length;
        Console.WriteLine ("Element at 1:  {0}", val);
        Console.WriteLine ("Dimension:     {0}", dim);
        Console.WriteLine ("Length:        {0}", len);

        /** testing operators */
        Console.Write ("Vector v1: ");
        double[] values1 = { 2.0, 4.0, 6.0 };
        v1 = new Vector (values1);
        v1.Print ();
        Console.WriteLine ();

        Console.Write ("Vector v2: ");
        double[] values2 = { 1.0, 2.0, 3.0 };
        v2 = new Vector (values2);
        v2.Print ();
        Console.WriteLine ();

        Console.Write ("v1 * 5:    ");
        v3 = v1 * 5;
        v3.Print ();
        Console.WriteLine ();

        Console.Write ("10 * v1:   ");
        v3 = 10 * v1;
        v3.Print ();
        Console.WriteLine ();

        Console.Write ("v1 + v1:   ");
        v3 = v1 + v1;
        v3.Print ();
        Console.WriteLine ();

        Console.Write ("v1 * v1:   ");
        double n = v1 * v1;
        Console.Write ("{0}", n);
        Console.WriteLine ();

        Console.Write ("v1 == v2:  ");
        bool b = v1 == v2;
        Console.Write ("{0}", b);
        Console.WriteLine ();

        Console.Write ("v1 == v1:  ");
        b = v1 == v1;
        Console.Write ("{0}", b);
        Console.WriteLine ();
    }
}
