#define VARIANTE_WITHOUT_STRING
// #define VARIANTE_WITH_STRING

#if VARIANTE_WITHOUT_STRING
using System;
using System.Text;

class Account
{
    private int accountNumber;
    protected decimal balance;

    public Account (int accountNumber)
    {
        this.accountNumber = accountNumber;
        this.balance = 0.00M;
    }

    public int AccountNumber
    {
        get
        {
            return this.accountNumber;
        }
    }

    public decimal Balance
    {
        get
        {
            return this.balance;
        }
    }

    public bool Deposit (decimal amount)
    {
        this.balance += amount;
        return true;
    }

    public virtual bool Withdrawal (decimal amount)
    {
        if (amount > this.balance)
            return false;

        this.balance -= amount;
        return true;
    }

    public override String ToString ()
    {
        return String.Format ("Account {0}: Deposit={1:.00}",
            this.accountNumber, this.balance);
    }    
}

class CreditAccount : Account
{
    private decimal limit;

    public CreditAccount (int accountNumber, decimal limit)
        : base (accountNumber)
    {
        this.limit = limit;
    }

    public decimal Limit
    {
        get
        {
            return this.limit;
        }
    }

    public override bool Withdrawal (decimal amount)
    {
        if (this.balance + this.limit - amount < 0.0M)
            return false;

        this.balance -= amount;
        return true;
    }

    public override String ToString ()
    {
        return String.Format ("{0} Limit={1}", base.ToString (), this.limit);
    }    
}

class DepositAccount : Account
{
    private decimal interestRate;

    public DepositAccount (int accountNumber, decimal interestRate)
        : base (accountNumber)
    {
        this.interestRate = interestRate;
    }

    public decimal InterestRate
    {
        get
        {
            return this.interestRate;
        }
    }

    public void ComputeInterest (int numDays)
    {
        decimal interest =
            (numDays * this.interestRate * this.balance) / (365 * 100);
        this.balance += interest;
    }

    public override String ToString ()
    {
        return String.Format ("{0} InterestRate={1}",
            base.ToString (), this.interestRate);
    }    
}

class Bank
{
    private Account[] accounts;

    public Bank () : this (100) {}

    public Bank (int capacity)
    {
        this.accounts = new Account[capacity];
    }

    public bool CreateAccount (out int accountNumber)
    {
        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] == null)
            {
                this.accounts[i] = new Account (i);
                accountNumber = i;
                return true;
            }
        }

        accountNumber = -1;
        return false;
    }

    public bool CreateCreditAccount (decimal limit, out int accountNumber)
    {
        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] == null)
            {
                this.accounts[i] = new CreditAccount (i, limit);
                accountNumber = i;
                return true;
            }
        }

        accountNumber = -1;
        return false;
    }

    public bool CreateDepositAccount (decimal interestRate, out int accountNumber)
    {
        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] == null)
            {
                this.accounts[i] = new DepositAccount (i, interestRate);
                accountNumber = i;
                return true;
            }
        }

        accountNumber = -1;
        return false;
    }

    // allow array-like access for accounts
    public Account this [int accountNumber]
    {
        get
        {
            if (accountNumber < 0 || accountNumber >= accounts.Length)
                return null;

            return accounts[accountNumber];
        }
    }

    public bool DeleteAccount (int accountNumber)
    {
        if (accountNumber < 0 || accountNumber >= accounts.Length)
            return false;

        if (this.accounts[accountNumber] == null)
            return false;

        this.accounts[accountNumber] = null;
        return true;
    }

    public decimal TotalBalance
    {
        get
        {
            decimal totalBalance = 0;
            for (int i = 0; i < accounts.Length; i ++)
                if (this.accounts[i] != null)
                    totalBalance += this.accounts[i].Balance;

            return totalBalance;
        }
    }

    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder (512);
        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] != null)
                sb.AppendFormat ("{0}\n", this.accounts[i].ToString ());
        }
        return sb.ToString ();
    }
}

class Demo
{
    public static void Main ()
    {
        // testing class Account
        Account ba = new Account (1001);
        ba.Deposit (100.0M);
        ba.Withdrawal (50.0M);
        Console.WriteLine ("Account: {0}", ba.ToString ());

        // testing class CreditAccount
        CreditAccount dba = new CreditAccount (1002, 100.0M);
        dba.Deposit (100.0M);
        dba.Withdrawal (200.0M);
        Console.WriteLine ("CreditAccount: {0}", dba.ToString ());

        // testing class DepositAccount
        DepositAccount sba = new DepositAccount (1003, 5.0M);
        sba.Deposit (100.0M);
        sba.ComputeInterest (365);
        Console.WriteLine ("DepositAccount: {0}", sba.ToString ());
        sba.ComputeInterest (100);
        Console.WriteLine ("DepositAccount: {0}", sba.ToString ());

        // testing class Bank
        Bank b = new Bank ();

        // create and use an account
        int accountNumber;
        b.CreateAccount (out accountNumber);

        // use this account
        Account ba1 = b[accountNumber];
        for (int i = 0; i < 10; i ++)
            ba1.Deposit (100 * i);
        Console.WriteLine ("Current balance: {0}", ba1.Balance);

        // create and use an drawing account
        b.CreateCreditAccount (500.0M, out accountNumber);
        Account ba2 = b[accountNumber];
        for (int i = 0; i < 10; i ++)
            ba2.Deposit (100 * i);
        Console.WriteLine ("Current balance: {0}", ba2.Balance);

        // create and use a savings account
        b.CreateDepositAccount (500.0M, out accountNumber);
        Account ba3 = b[accountNumber];
        for (int i = 0; i < 10; i ++)
            ba3.Deposit (100 * i);
        Console.WriteLine ("Current balance: {0}", ba3.Balance);

        // obtain current balance of bank
        Console.WriteLine ("Current balance: {0}", b.TotalBalance);

        // testing ToString method
        Console.WriteLine ("{0}", b);
    }
}
#endif

#if VARIANTE_WITH_STRING
using System;
using System.Text;

class Account
{
    private String name;
    private int accountNumber;
    protected Decimal balance;

    public Account (String name, int accountNumber)
    {
        this.name = name;
        this.accountNumber = accountNumber;
        this.balance = 0.00M;
    }

    public String Name
    {
        get
        {
            return this.name;
        }
    }

    public int AccountNumber
    {
        get
        {
            return this.accountNumber;
        }
    }

    public Decimal Balance
    {
        get
        {
            return this.balance;
        }
    }

    public bool Deposit (Decimal amount)
    {
        this.balance += amount;
        return true;
    }

    public virtual bool Withdrawal (Decimal amount)
    {
        if (amount > this.balance)
            return false;

        this.balance -= amount;
        return true;
    }

    public override String ToString ()
    {
        return String.Format ("{0} (Account {1}): Deposit={2:.00}",
            this.name, this.accountNumber, this.balance);
    }    
}

class CreditAccount : Account
{
    private decimal limit;

    public CreditAccount (String name, int accountNumber, Decimal limit) :
        base (name, accountNumber)
    {
        this.limit = limit;
    }

    public Decimal Limit
    {
        get
        {
            return this.limit;
        }
    }

    public override bool Withdrawal (Decimal amount)
    {
        if (this.balance + this.limit - amount < 0.0M)
            return false;

        return base.Withdrawal (amount);
    }

    public override String ToString ()
    {
        return String.Format ("{0} Limit={1}", base.ToString (), this.limit);
    }    
}

class DepositAccount : Account
{
    private decimal interestRate;

    public DepositAccount (
        String name, int accountNumber, Decimal interestRate) :
        base (name, accountNumber)
    {
        this.interestRate = interestRate;
    }

    public Decimal InterestRate
    {
        get
        {
            return this.interestRate;
        }
    }

    public void ComputeInterest (int numDays)
    {
        Decimal interest =
            (numDays * this.interestRate * this.balance) / (365 * 100);
        this.balance += interest;
    }

    public override String ToString ()
    {
        String s = String.Format (
            "{0} InterestRate={1}", base.ToString (), this.interestRate
            );
        return s;
    }    
}

class Bank
{
    private String name;
    private Account[] accounts;

    public Bank (String name) : this (name, 100)
    {
    }

    public Bank (String name, int capacity)
    {
        this.name = name;
        this.accounts = new Account[capacity];
    }

    public bool CreateAccount (String name, out int accountNumber)
    {
        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] == null)
            {
                this.accounts[i] = new Account (name, i);
                accountNumber = i;
                return true;
            }
        }

        accountNumber = -1;
        return false;
    }

    public bool CreateCreditAccount (String name, Decimal limit, out int accountNumber)
    {
        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] == null)
            {
                this.accounts[i] = new CreditAccount (name, i, limit);
                accountNumber = i;
                return true;
            }
        }

        accountNumber = -1;
        return false;
    }

    public bool CreateDepositAccount (String name, Decimal interestRate, out int accountNumber)
    {
        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] == null)
            {
                this.accounts[i] = new DepositAccount (name, i, interestRate);
                accountNumber = i;
                return true;
            }
        }

        accountNumber = -1;
        return false;
    }

    // allow array-like access for accounts
    public Account this [int accountNumber]
    {
        get
        {
            if (accountNumber < 0 || accountNumber >= accounts.Length)
                return null;

            return accounts[accountNumber];
        }
    }

    public bool DeleteAccount (int accountNumber)
    {
        if (accountNumber < 0 || accountNumber >= accounts.Length)
            return false;

        if (this.accounts[accountNumber] == null)
            return false;

        this.accounts[accountNumber] = null;
        return true;
    }

    public Decimal TotalBalance
    {
        get
        {
            Decimal totalBalance = 0;
            for (int i = 0; i < accounts.Length; i ++)
                if (this.accounts[i] != null)
                    totalBalance += this.accounts[i].Balance;

            return totalBalance;
        }
    }

    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder (512);
        sb.AppendFormat ("Name: {0}", name);
        sb.Append("\n");

        for (int i = 0; i < accounts.Length; i ++)
        {
            if (this.accounts[i] != null)
                sb.AppendFormat ("{0}\n", this.accounts[i].ToString ());
        }

        return sb.ToString ();
    }
}

class Demo
{
    public static void Main ()
    {
        // testing class Account
        Account ba = new Account ("Bob Fisher", 1);
        ba.Deposit (100.0M);
        ba.Withdrawal (50.0M);
        Console.WriteLine ("Account: {0}", ba.ToString ());

        // testing class CreditAccount
        CreditAccount dba =
            new CreditAccount ("Bill Miller", 2, 100.0M);
        dba.Deposit (100.0M);
        dba.Withdrawal (200.0M);
        Console.WriteLine ("CreditAccount: {0}", dba.ToString ());

        // testing class DepositAccount
        DepositAccount sba =
            new DepositAccount ("Jack Clark", 3, 5.0M);
        sba.Deposit (100.0M);
        sba.ComputeInterest (365);
        Console.WriteLine ("DepositAccount: {0}", sba.ToString ());
        sba.ComputeInterest (100);
        Console.WriteLine ("DepositAccount: {0}", sba.ToString ());

        // testing class Bank
        Bank b = new Bank ("Savings");

        // create and use an account
        int accountNumber;
        b.CreateAccount ("Dave Valentine", out accountNumber);

        // use this account
        Account ba1 = b[accountNumber];
        for (int i = 0; i < 10; i ++)
            ba1.Deposit (100 * i);
        Console.WriteLine ("Current balance: {0}", ba1.Balance);

        // create and use an drawing account
        b.CreateCreditAccount ("Dave Valentine",
            500.0M, out accountNumber);
        Account ba2 = b[accountNumber];
        for (int i = 0; i < 10; i ++)
            ba2.Deposit (100 * i);
        Console.WriteLine ("Current balance: {0}", ba2.Balance);

        // create and use a savings account
        b.CreateDepositAccount ("Bill Connor",
            500.0M, out accountNumber);
        Account ba3 = b[accountNumber];
        for (int i = 0; i < 10; i ++)
            ba3.Deposit (100 * i);
        Console.WriteLine ("Current balance: {0}", ba3.Balance);

        // obtain current balance of bank
        Console.WriteLine ("Current balance: {0}", b.TotalBalance);

        // testing ToString method
        Console.WriteLine ("{0}", b);
    }
}
#endif


