#define VARIANTE_01
// #define VARIANTE_02

using System;
using System.Collections;

#if VARIANTE_01
class Matrix : ICloneable
{
    protected int rows;
    protected int cols;
    protected int[,] values;

    // default c'tor: construct a 1-by-1 matrix
    public Matrix () : this (1,1)
    {
    }

    // user defined c'tor: construct a matrix of zeros
    public Matrix (int rows, int cols)
    {
        this.rows = rows;
        this.cols = cols;
        this.values = new int[rows, cols];
    }

    // user defined c'tor: construct a matrix from a 2-D array
    public Matrix (int[,] values)
    {
        this.rows = values.GetLength (0);
        this.cols = values.GetLength (1);
        this.values = new int[rows, cols];

        // make a deep copy of the matrix elements
        for (int r = 0; r < this.rows; r ++)
            for (int c = 0; c < this.cols; c ++)
                this.values[r, c] = values[r, c];
    }

    // row dimension property
    public int RowCount
    {
        get
        {
            return this.rows;
        }
    }

    // column dimension property
    public int ColCount
    {
        get
        {
            return this.cols;
        }
    }

    // standard methods
    public Matrix Add (Matrix m)
    {
        Matrix temp = new Matrix (this.rows, this.cols);

        for (int r = 0; r < this.rows; r ++)
            for (int c = 0; c < this.cols; c ++)
                temp.values[r, c] = this.values[r, c] + m.values[r, c];

        return temp;    
    }

    public Matrix Sub (Matrix m)
    {
        Matrix temp = new Matrix (this.rows, this.cols);

        for (int r = 0; r < this.rows; r ++)
            for (int c = 0; c < this.cols; c ++)
                temp.values[r, c] = this.values[r, c] - m.values[r, c];

        return temp;    
    }

    public Matrix Mul (Matrix m)
    {
        Matrix temp = new Matrix (this.rows, m.cols);

        for (int r = 0; r < temp.RowCount; r ++)
        {
            for (int c = 0; c < temp.ColCount; c ++)
            {
                int val = 0;
                for (int k = 0; k < this.cols; k ++)
                    val += this.values[r, k] * m.values[k, c];

                temp.values[r, c] = val;
            }
        }

        return temp;    
    }

    // implementation of interface ICloneable (deep copy)
    public Object Clone ()
    {
        Matrix temp = new Matrix (this.rows, this.cols);

        for (int r = 0; r < this.rows; r ++)
            for (int c = 0; c < this.cols; c ++)
                temp.values[r, c] = this.values[r, c];

        return temp;
    }

    // modifying contract with base class 'Object'
    public override bool Equals (Object o)
    {
        if (o == null)
            return false;

        Type typ = this.GetType ();
        if (! typ.Equals (o.GetType ()))
            return false;

        Matrix m = (Matrix) o;
        if (this.rows != m.rows)
            return false;

        if (this.cols != m.cols)
            return false;

        for (int r = 0; r < this.rows; r ++)
            for (int c = 0; c < this.cols; c ++)
                if (this.values[r, c] != m.values[r, c])
                    return false;

        return true;
    }

    public override int GetHashCode ()
    {
        return this.values.GetHashCode ();
    }

    // operators
    public static Matrix operator + (Matrix x, Matrix y)
    {
        return x.Add (y);
    }

    public static Matrix operator - (Matrix x, Matrix y)
    {
        return x.Sub (y);
    }

    public static Matrix operator * (Matrix x, Matrix y)
    {
        return x.Mul (y);
    }

    public static bool operator == (Matrix x, Matrix y)
    {
        return x.Equals (y);
    }

    public static bool operator != (Matrix x, Matrix y)
    {
        return !(x == y);
    }

    // indexer (read/write access to matrix elements)
    public int this[int x, int y]
    {
        get
        {
            return this.values[x,y];
        }

        set
        {
            this.values[x,y] = value;
        }
    }

    // miscellaneous methods
    public Matrix SubMatrix (int x, int y)
    {
        Matrix temp = new Matrix (this.rows - 1, this.cols - 1);

        // II. quadrant
        for (int r = 0; r < x; r ++)
            for (int c = 0; c < y; c ++)
                temp.values[r, c] = this.values[r, c];

        // I. quadrant
        for (int r = 0; r < x; r ++)
            for (int c = y; c < this.cols - 1; c ++)
                temp.values[r, c] = this.values[r, c + 1];

        // III. quadrant
        for (int r = x; r < this.rows - 1; r ++)
            for (int c = 0; c < y; c ++)
                temp.values[r, c] = this.values[r + 1, c];

        // IV. quadrant
        for (int r = x; r < this.rows - 1; r ++)
            for (int c = y; c < this.cols - 1; c ++)
                temp.values[r, c] = this.values[r + 1, c + 1];

        return temp;
    }

    public int Determinant ()
    {
        if (this.rows == 1 && this.cols == 1)
            return this.values[0,0];
        else
        {
            int det = 0;
            int sign = 1;
            for (int k = 0; k < this.rows; k ++)
            {
                Matrix sub = this.SubMatrix (k, 0);
                det += sign * this.values[k, 0] * sub.Determinant ();
                sign = - sign;
            }

            return det;
        }
    }

    public Matrix Transpose ()
    {
        Matrix temp = new Matrix (this.cols, this.rows);

        for (int r = 0; r < rows; r ++)
            for (int c = 0; c < cols; c ++)
                temp.values[c, r] = this.values[r, c];

        return temp;
    }

    public static Matrix Identity (int rows, int cols)
    {
        Matrix temp = new Matrix (rows, cols);

        for (int r = 0; r < rows; r ++)
            for (int c = 0; c < cols; c ++)
                temp.values [r, c] = (r == c) ? 1 : 0;

        return temp;    
    }    

    public void Print ()
    {
        for (int r = 0; r < this.rows; r ++)
        {
            for (int c = 0; c < this.cols; c ++)
                Console.Write ("{0, 5}", this.values[r, c]);
            Console.WriteLine ();
        }
    }
}
#endif

#if VARIANTE_02
enum MatrixError
{
    NoError,
    InvalidRowIndex,
    InvalidColumnIndex,
    InvalidRowDimension,
    InvalidColumnDimension
}

class MatrixException : ApplicationException
{
    private int errorValue;
    private MatrixError kind;

    public MatrixException () : base ("Matrix Error")
    {
        this.errorValue = 0;
        this.kind = MatrixError.NoError;
    }    

    public MatrixError Kind
    {
        set
        {
            this.kind = value;
        }

        get
        {
            return this.kind;
        }
    }

    public int ErrorValue
    {
        set
        {
            this.errorValue = value;
        }

        get
        {
            return this.errorValue;
        }
    }
}

class Matrix
{
    protected int rows;
    protected int cols;
    protected int[,] values;

    /** c'tors */

    // user defined c'tor: construct a matrix of zeros
    public Matrix (int rows, int cols)
    {
        this.rows = rows;
        this.cols = cols;
        this.values = new int[rows, cols];
    }

    // indexer: read/write access to matrix elements
    public int this[int x, int y]
    {
        get
        {
            if (x < 0 || x >= this.rows)
            {
                MatrixException me = new MatrixException ();
                me.Kind = MatrixError.InvalidRowIndex;
                me.ErrorValue = x;
                throw me;
            }

            if (y < 0 || y >= this.cols)
            {
                MatrixException me = new MatrixException ();
                me.Kind = MatrixError.InvalidColumnIndex;
                me.ErrorValue = y;
                throw me;
            }
            
            return this.values[x,y];
        }

        set
        {
            if (x < 0 || x >= this.rows)
            {
                MatrixException me = new MatrixException ();
                me.Kind = MatrixError.InvalidRowIndex;
                me.ErrorValue = x;
                throw me;
            }

            if (y < 0 || y >= this.cols)
            {
                MatrixException me = new MatrixException ();
                me.Kind = MatrixError.InvalidColumnIndex;
                me.ErrorValue = y;
                throw me;
            }
            
            this.values[x,y] = value;
        }
    }

    // matrix addition
    public Matrix Add (Matrix m)
    {
        if (this.rows != m.rows)
        {
            MatrixException me = new MatrixException ();
            me.Kind = MatrixError.InvalidRowDimension;
            me.ErrorValue = m.rows;
            throw me;
        }

        if (this.cols != m.cols)
        {
            MatrixException me = new MatrixException ();
            me.Kind = MatrixError.InvalidColumnDimension;
            me.ErrorValue = m.cols;
            throw me;
        }

        Matrix temp = new Matrix (this.rows, this.cols);

        for (int r = 0; r < this.rows; r ++)
            for (int c = 0; c < this.cols; c ++)
                temp.values[r, c] = this.values[r, c] + m.values[r, c];

        return temp;
    }
}
#endif
