using System;
using System.Text;

class Coefficients : IComparable
{
    private double[] coefficients;

    public Coefficients ()
    {
        // create zero polynom
        this.coefficients = new double[1];
        this.coefficients[0] = 0.0;
    }

    public Coefficients (int rank)
    {
        // create polynom with specified rank
        this.coefficients = new double [rank + 1];
    }

    public Coefficients (double[] coefficients)
    {
        // remove leading zeros
        int max = coefficients.Length - 1;
        while (max > 0 && coefficients[max] == 0)
            max --;

        // create polynom with specified coefficients
        this.coefficients = new double[max + 1];
        for (int i = 0; i < this.coefficients.Length; i ++)
        {
            this.coefficients[i] = coefficients[i];
        }
    }
    
    public int Rank
    {
        get
        {
            return this.coefficients.Length - 1;
        }
    }
    
    public double this [int index]
    {
        get
        {
            if (index <= this.Rank)
                return this.coefficients[index];
            else
                return 0.0;  // leading zero
        }

        set
        {
            if (index <= this.Rank)
                this.coefficients[index] = value;
        }
    }

    // implementation of interface 'IComparable'
    public int CompareTo (Object o)
    {
        Coefficients c = (Coefficients) o;
        if (this.Rank < c.Rank)
            return -1;
        if (this.Rank > c.Rank)
            return 1;

        for (int i = this.Rank; i >= 0; i --)
        {
            if (this.coefficients[i] < c.coefficients[i])
                return -1;
            if (this.coefficients[i] > c.coefficients[i])
                return 1;
        }
        return 0;
    }
}

class Polynom : ICloneable, IComparable
{
    private Coefficients coefficients;

    // c'tors
    public Polynom ()
    {
        // create zero polynom
        this.coefficients = new Coefficients ();
    }

    public Polynom (double[] values)
    {
        this.coefficients = new Coefficients (values);
    }

    // private c'tor
    private Polynom (int rank)
    {
        this.coefficients = new Coefficients (rank);
    }

    // properties
    public int Rank
    {
        get
        {
            return this.coefficients.Rank;
        }
    }

    public Coefficients Coefficients
    {
        get
        {
            return this.coefficients;
        }
    }

    // operators
    public static Polynom operator+ (Polynom p1, Polynom p2)
    {
        int rank = Math.Max (p1.Rank, p2.Rank);
        Polynom p = new Polynom (rank);

        for (int i = 0; i <= rank; i ++)
            p.Coefficients[i] = p1.Coefficients[i] + p2.Coefficients[i];

        return p;
    }

    public static Polynom operator* (Polynom p1, Polynom p2)
    {
        Polynom q = new Polynom ();
        for (int i = 0; i <= p1.Rank; i ++)
        {
            Polynom p = (Polynom) p2.Clone ();
            p.MulConstant (p1.Coefficients[i]);
            p.MulX (i);
            q += p;
        }

        return q;
    }
    
    // private helper methods
    private void MulConstant (double c)
    {
        for (int i = 0; i <= this.Coefficients.Rank; i ++)
        {
            this.Coefficients[i] *= c;
        }
    }

    private void MulX ()
    {
        double[] values = new double[this.Rank + 2];
        for (int i = 1; i < values.Length; i ++)
            values[i] = this.Coefficients[i-1];
        this.coefficients = new Coefficients (values);
    }

    private void MulX (int k)
    {
        for (int i = 0; i < k; i ++) 
            this.MulX ();
    }

    // modifying contract with base class 'Object'
    public override bool Equals (Object o)
    {
        if (o == null)
            return false;
    
        Type typ = this.GetType ();
        if (! typ.Equals (o.GetType ()))
            return false;
    
        Polynom p = (Polynom) o;
        if (this.Rank != p.Rank)
            return false;

        for (int i = 0; i <= this.Rank; i ++)
            if (this.Coefficients[i] != p.Coefficients[i])
                return false;

        return true;
    }

    public override int GetHashCode ()
    {
        return this.Coefficients.GetHashCode ();
    }
    
    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder (124);

        sb.Append (this.ToString (this.Coefficients.Rank, false));
        for (int i = this.Coefficients.Rank - 1; i >= 0; i --)
            sb.Append (this.ToString (i, true));

        return sb.ToString ();
    }

    // private helper method
    private String ToString (int i, bool leadingSign)
    {
        StringBuilder sb = new StringBuilder (32);
        double d = Math.Abs (this.Coefficients[i]);

        if (this.Coefficients[i] >= 0)
        {
            if (leadingSign)
                sb.Append (" + ");
        }
        else
            sb.Append (" - ");

        sb.Append (d.ToString ());

        if (i > 0) 
        {
            sb.Append ("*x");
            if (i > 1) 
            {
                sb.Append (String.Format ("^{0}", i));
            }
        }

        return sb.ToString ();
    }

    // private helper method
    private double Compute (double x)
    {
        double y = this.Coefficients[this.Rank];        
        for (int i = this.Rank - 1; i >= 0; i--) 
            y = this.Coefficients[i] + y * x;
        return y;
    }

    // compute value
    public double this [double x]
    {
        get
        {
            return this.Compute (x);
        }
    }

    // implementation of interface 'ICloneable'
    public Object Clone ()
    {
        Polynom p = new Polynom (this.Rank);

        for (int i = 0; i <= this.Rank; i ++)
            p.Coefficients[i] = this.Coefficients[i];

        return p;
    }

    // implementation of interface 'IComparable'
    public int CompareTo (Object o)
    {
        Polynom p = (Polynom) o;
        return this.Coefficients.CompareTo (p.Coefficients);
    }

    // comparison operators
    public static bool operator== (Polynom a, Polynom b)
    {
        return a.CompareTo (b) == 0;
    }

    public static bool operator!= (Polynom a, Polynom b)
    {
        return ! (a == b);
    }

    public static bool operator< (Polynom a, Polynom b)
    {

        return a.CompareTo (b) == -1;
    }

    public static bool operator<= (Polynom a, Polynom b)
    {
        return (a < b) || (a == b);
    }

    public static bool operator> (Polynom a, Polynom b)
    {
        return ! (a <= b);
    }

    public static bool operator>= (Polynom a, Polynom b)
    {
        return (a > b) || (a == b);
    }
}
