#define STRING_VARIANT
// #define INTEGER_VARIANT

using System;
using System.Text;

#if STRING_VARIANT
class BigInteger : IComparable, ICloneable
{
    private StringBuilder digits;

    // c'tor(s)
    private BigInteger (StringBuilder digits)
    {
        this.digits = digits;
    }
    
    public BigInteger (String number)
    {
        this.digits = new StringBuilder (number.Length); 

        // copy string into internal array in reverse order
        for (int i = number.Length - 1; i >= 0; i --)
        {
            if (number[i] == '+' || number[i] == '.')
                continue;

            this.digits.Append (number[i]);
        }
    }

    public BigInteger (int n) : this (n.ToString ())
    {
    }

    public BigInteger (long n) : this (n.ToString ())
    {
    }
    
    // indexer
    private char this[int i]
    {
        get
        {
            if (i < this.digits.Length)
                return this.digits[i];
            else
                return '0';  // leading zero
        }
    }

    // properties
    public int Cardinality
    {
        get
        {
            return this.digits.Length;
        }
    }

    // conversion operators
    public static implicit operator BigInteger (int n)
    {
        return new BigInteger (n.ToString ());
    }

    public static implicit operator BigInteger (long n)
    {
        return new BigInteger (n.ToString ());
    }

    // arithmetic operators
    public static BigInteger operator+ (BigInteger a, BigInteger b)
    {
        int maxDigits = Math.Max (a.digits.Length, b.digits.Length);
        StringBuilder tmp = new StringBuilder (maxDigits);
        
        int carry = 0;
        for (int i = 0; i < maxDigits; i ++)
        {
            int sum = (a[i] - '0') + (b[i] - '0') + carry;
            carry = (sum >= 10) ? 1 : 0;
            tmp.Append ((char) ((sum % 10) + '0'));
        }
        if (carry > 0)
            tmp.Append ((char) (carry + '0'));

        return new BigInteger (tmp);
    }

    public static BigInteger operator* (BigInteger a, int digit)
    {
        StringBuilder tmp = new StringBuilder ();

        int carry = 0;
        for (int i = 0; i < a.digits.Length; i ++)
        {
            int product = digit * (a[i] - '0') + carry;
            carry = product / 10;
            tmp.Append ((char) ((product % 10) + '0'));
        }
        if (carry > 0)
            tmp.Append ((char) (carry + '0'));

        return new BigInteger (tmp);
    }

    public static BigInteger operator* (BigInteger a, BigInteger b)
    {
        // apply algorithm of "stellenweise Multiplikation"
        BigInteger result = new BigInteger (0);
        for (int i = 0; i < b.Cardinality; i ++)
            result += (a * (b[i] - '0')).PowerOfTen (i);

        return result;
    }

    public static BigInteger operator++ (BigInteger a)
    {
        BigInteger tmp = (BigInteger) a.Clone ();
        tmp = tmp + 1;
        return tmp;
    }

    // comparison operators
    public static bool operator== (BigInteger a, BigInteger b)
    {
        if (a.digits.Length != b.digits.Length)
            return false;

        return a.CompareTo (b) == 0;
    }

    public static bool operator== (BigInteger a, String s)
    {
        BigInteger tmp = new BigInteger (s);
        return a == tmp;
    }
    
    public static bool operator!= (BigInteger a, String s)
    {
        BigInteger tmp = new BigInteger (s);
        return a != tmp;
    }    

    public static bool operator== (String s, BigInteger a)
    {
        return a == s;
    }

    public static bool operator!= (String s, BigInteger a)
    {
        return a != s;
    }

    public static bool operator!= (BigInteger a, BigInteger b)
    {
        return ! (a == b);
    }

    public static bool operator< (BigInteger a, BigInteger b)
    {

        return a.CompareTo (b) == -1;
    }

    public static bool operator<= (BigInteger a, BigInteger b)
    {
        return (a < b) || (a == b);
    }

    public static bool operator> (BigInteger a, BigInteger b)
    {
        return ! (a <= b);
    }

    public static bool operator>= (BigInteger a, BigInteger b)
    {
        return (a > b) || (a == b);
    }

    // methods
    public BigInteger Power (int exp)
    {
        if (exp == 0)
            return new BigInteger ("1");

        BigInteger tmp = (BigInteger) this.Clone ();
        for (int i = 1; i < exp; i ++)
            tmp *= this;

        return tmp;
    }

    // helper methods
    private BigInteger PowerOfTen (int factor)
    {
        if (factor == 0)
            return this;

        StringBuilder tmp = new StringBuilder (this.Cardinality + factor);
        for (int i = 0; i < factor; i ++)
            tmp.Append ('0');

        for (int i = 0; i < this.Cardinality; i ++)
            tmp.Append (this[i]);

        return new BigInteger (tmp);
    }

    // implementation of interface 'IComparable'
    public int CompareTo (Object o)
    {
        BigInteger tmp = (BigInteger) o;
        if (this.digits.Length < tmp.digits.Length)
            return -1;
        if (this.digits.Length > tmp.digits.Length)
            return 1;

        for (int i = this.digits.Length - 1; i >= 0; i --)
        {
            if (this.digits[i] < tmp.digits[i])
                return -1;
            if (this.digits[i] > tmp.digits[i])
                return 1;
        }
        return 0;
    }

    // implementation of interface 'ICloneable'
    public Object Clone ()
    {
        String s = this.digits.ToString ();
        char[] digits = s.ToCharArray ();
        Array.Reverse (digits);
        return new BigInteger (new String (digits));
    }

    // modifying contract with base class 'Object'
    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder ();
        for (int i = 0; i < this.digits.Length; i ++)
        {
            sb.Insert (0, this.digits[i]);
            if (i < this.digits.Length - 1 && (i + 1) % 3 == 0)
                sb.Insert (0, '.');
        }
        return sb.ToString ();
    }

    public override bool Equals (Object o)
    {
        return this.CompareTo ((BigInteger) o) == 0;
    }

    public override int GetHashCode()
    {
        return this.digits.GetHashCode ();
    }
}
#endif


#if INTEGER_VARIANT
class BigInteger : IComparable, ICloneable
{
    private int[] digits;
    private int numDigits;

    // c'tor(s)
    private BigInteger ()
    {
        this.numDigits = 0;
        this.digits = new int[16];
    }

    public BigInteger (String number) : this ()
    {
        // copy string into internal array in reverse order
        for (int i = number.Length - 1; i >= 0; i --)
        {
            if (number[i] == '+' || number[i] == '.')
                continue;

            this.AddDigit (number[i] - '0');
        }
    }

    public BigInteger (int n) : this (n.ToString ())
    {
    }

    public BigInteger (long n) : this (n.ToString ())
    {
    }
    
    // indexer
    private int this[int i]
    {
        get
        {
            if (i < this.numDigits)
                return this.digits[i];
            else
                return 0;  // leading zero
        }

        set
        {
            if (i < this.numDigits)
                this.digits[i] = value;
        }
    }

    // properties
    public int Cardinality
    {
        get
        {
            return this.numDigits;
        }
    }

    // conversion operator
    public static implicit operator BigInteger (int n)
    {
        String s = n.ToString ();
        return new BigInteger (s);
    }

    // arithmetic operators
    public static BigInteger operator+ (BigInteger a, BigInteger b)
    {
        BigInteger result = new BigInteger ();

        int maxDigits = Math.Max (a.numDigits, b.numDigits);

        int carry = 0;
        for (int i = 0; i < maxDigits; i ++)
        {
            int sum = a[i] + b[i] + carry;
            carry = (sum >= 10) ? 1 : 0;
            result.AddDigit (sum  % 10);
        }
        if (carry > 0)
            result.AddDigit (carry);
        return result;
    }

    public static BigInteger operator* (BigInteger a, int digit)
    {
        BigInteger result = new BigInteger ();

        int carry = 0;
        for (int i = 0; i < a.numDigits; i ++)
        {
            int product = digit * a[i] + carry;
            carry = product / 10;
            result.AddDigit (product % 10);
        }
        if (carry > 0)
            result.AddDigit (carry);
        return result;
    }

    public static BigInteger operator* (BigInteger a, BigInteger b)
    {
        // apply algorithm of "stellenweise Multiplikation"
        BigInteger result = new BigInteger (0);
        for (int i = 0; i < b.Cardinality; i ++)
            result += (a * b[i]).PowerOfTen (i);

        return result;
    }
    
    public static BigInteger operator++ (BigInteger a)
    {
        BigInteger tmp = (BigInteger) a.Clone ();
        tmp = tmp + 1;
        return tmp;
    }

    public static bool operator== (BigInteger a, BigInteger b)
    {
        if (a.numDigits != b.numDigits)
            return false;

        for (int i = 0; i < a.numDigits; i ++)
            if (a.digits[i] != b.digits[i])
                return false;

        return true;
    }

    public static bool operator== (BigInteger a, String s)
    {
        BigInteger tmp = new BigInteger (s);
        return a == tmp;
    }
    
    public static bool operator!= (BigInteger a, String s)
    {
        BigInteger tmp = new BigInteger (s);
        return a != tmp;
    }    

    public static bool operator== (String s, BigInteger a)
    {
        return a == s;
    }

    public static bool operator!= (String s, BigInteger a)
    {
        return a != s;
    }

    public static bool operator!= (BigInteger a, BigInteger b)
    {
        return ! (a == b);
    }

    public static bool operator< (BigInteger a, BigInteger b)
    {

        return a.CompareTo (b) == -1;
    }

    public static bool operator<= (BigInteger a, BigInteger b)
    {
        return (a < b) || (a == b);
    }

    public static bool operator> (BigInteger a, BigInteger b)
    {
        return ! (a <= b);
    }

    public static bool operator>= (BigInteger a, BigInteger b)
    {
        return (a > b) || (a == b);
    }

    // methods
    public BigInteger Power (int exp)
    {
        if (exp == 0)
            return new BigInteger ("1");

        BigInteger tmp = (BigInteger) this.Clone ();
        for (int i = 1; i < exp; i ++)
            tmp *= this;

        return tmp;
    }
 
    // helper methods
    private void AddDigit (int digit)
    {
        if (this.numDigits == this.digits.Length)
        {
            // allocate new array
            int[] tmp = new int[this.numDigits * 2];
            for (int i = 0; i < this.numDigits; i ++)
                tmp[i] = this.digits[i];
            this.digits = tmp;
        }

        // copy digit at end of array
        this.digits[this.numDigits] = digit;
        this.numDigits ++;
    }

    private BigInteger PowerOfTen (int factor)
    {
        if (factor == 0)
            return this;

        BigInteger tmp = new BigInteger ();
        tmp.digits = new int[this.Cardinality + factor];
        tmp.numDigits = this.numDigits + factor;

        for (int i = 0; i < factor; i ++)
            tmp.digits[i] = 0;

        for (int i = 0; i < this.Cardinality; i ++)
            tmp.digits[factor + i] = this.digits[i];

        return tmp;
    }

    // implementation of interface 'IComparable'
    public int CompareTo (Object o)
    {
        BigInteger tmp = (BigInteger) o;
        if (this.numDigits < tmp.numDigits)
            return -1;
        if (this.numDigits > tmp.numDigits)
            return 1;

        for (int i = this.numDigits - 1; i >= 0; i --)
        {
            if (this.digits[i] < tmp.digits[i])
                return -1;
            if (this.digits[i] > tmp.digits[i])
                return 1;
        }
        return 0;
    }

    // implementation of interface 'ICloneable'
    public Object Clone ()
    {
        BigInteger tmp = new BigInteger ();
        tmp.numDigits = this.numDigits;
        tmp.digits = (int[]) this.digits.Clone ();
        return tmp;
    }

    // modifying contract with base class 'Object'
    public override String ToString ()
    {
        StringBuilder sb = new StringBuilder ();
        for (int i = 0; i < this.numDigits; i ++)
        {
            sb.Insert (0, this.digits[i]);
            if (i < this.numDigits - 1 && (i + 1) % 3 == 0)
                sb.Insert (0, '.');
        }
        return sb.ToString ();
    }

    public override bool Equals (Object o)
    {
        return this.CompareTo ((BigInteger) o) == 0;
    }

    public override int GetHashCode()
    {
        return this.digits.GetHashCode ();
    }
}
#endif