using System;
using System.Drawing;
using System.Windows.Forms;

class MandelbrotForm : Form
{
    private MandelbrotExcerpt excerpt;

    public MandelbrotForm (int width, int height)
    {
        // setup form
        this.Text = "Mandelbrot";
        this.ResizeRedraw = true;
        this.ClientSize = new Size (width, height);

        // setup 'MandelbrotExcerpt' instance
        this.excerpt = new MandelbrotExcerpt (
            MandelbrotCoords.XMin,  // upper-left corner (x-coordinate)
            MandelbrotCoords.YMin,  // upper-left corner (y-coordinate)
            MandelbrotCoords.XWidth,  // width of complex number area
            MandelbrotCoords.YHeight  // height of complex number area)
        );
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // update 'MandelbrotExcerpt' instance
        this.excerpt.Width = this.ClientSize.Width - 1;
        this.excerpt.Height = this.ClientSize.Height - 1;
        
        // draw Mandelbrot set
        int start = Environment.TickCount;
        this.excerpt.Draw (e.Graphics);
        int end = Environment.TickCount - start;
        Console.WriteLine ("OnPaint: [{0} milliseconds]", end);
    }
}

class MandelbrotExcerpt
{
    private double xmin;    // upper-left corner (x-coordinate)
    private double ymin;    // upper-left corner (y-coordinate)
    private double xwidth;  // width of complex number area
    private double yheight; // height of complex number area
    private int width;      // total width of client area
    private int height;     // total height of client area

    public MandelbrotExcerpt (
        double xmin,
        double ymin,
        double xwidth,
        double yheight)
    {
        this.xmin = xmin;
        this.ymin = ymin;
        this.xwidth = xwidth;
        this.yheight = yheight;
    }

    public int Width
    {
        set
        {
            this.width = value;
        }
    }

    public int Height
    {
        set
        {
            this.height = value;
        }
    }
        
    public void Draw (Graphics g)
    {
        // create a default brush object
        SolidBrush brush = new SolidBrush (Color.Black);

        // apply Mandelbrot formula
        for (int x = 0; x <= this.width; x ++)
        {
            for (int y = 0; y <= this.height; y ++)
            {
                // convert pixel position into complex number
                double cx =
                    this.xmin + this.xwidth * ((double) x / this.width);
                double cy =
                    this.ymin + this.yheight * ((double) y / this.height);

                // compute numerical order
                double zx = 0.0;
                double zy = 0.0;
                int count = 0;
                while (true)
                {
                    count ++;

                    // apply Mandelbrot formula
                    double tmp = zx;
                    zx = (zx * zx) - (zy * zy) + cx;
                    zy = (2 * tmp * zy) + cy;

                    // Mandelbrot number found
                    if (( zx * zx) + (zy * zy) > 4)
                        break;

                    // it's not a Mandelbrot number
                    if (count >= MandelbrotColors.MaxIterations)
                        break;
                }

                // modify brush
                brush.Color = (count >= MandelbrotColors.MaxIterations) ?
                    Color.Black : MandelbrotColors.HSBColors[count];

                // draw pixel
                g.FillRectangle (brush, x, y, 1, 1);
            }
        }

        // release resources
        brush.Dispose ();
    }
}
