using System;
using System.Drawing;
using System.Windows.Forms;

delegate void GameStatusChangedEventHandler (
    Object sender, GameStatusChangedEventArgs e
);

enum GameStatus
{
    WhiteIsNext,
    BlackIsNext,
    WhiteHasWon,
    BlackHasWon
}

class GameStatusChangedEventArgs : EventArgs
{
    private GameStatus status;

    public GameStatusChangedEventArgs (GameStatus status)
    {
        this.status = status;
    }

    public GameStatus Status
    {
        get
        {
            return this.status;
        }
    }
}

class FourWinsPanel : Panel
{
    private const int MaxRow = 6;
    private const int MaxCol = 7;
    private const int Diameter = 50;

    // handling game status
    private GameStatus status;
    public event GameStatusChangedEventHandler GameStatusChanged;

    // game board
    private enum Stone { Empty, Black, White };
    private Stone[,] board;

    public FourWinsPanel ()
    {
        this.BorderStyle = BorderStyle.Fixed3D;

        this.status = GameStatus.WhiteIsNext;
        this.GameStatusChanged = null;

        this.board = new Stone[MaxRow, MaxCol];
        this.ResetBoard ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        this.DrawBoard (e.Graphics);
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        // enable registered clients to receive 'MouseDown' events
        base.OnMouseDown (e);

        // calculate cell
        int x = e.X;
        int y = e.Y;
        x -= 10;
        y -= 10;
        int row = MaxRow - 1 - y / Diameter;
        int col = x / Diameter;

        if (! this.IsValidClick (row, col))
            return;

        // update board
        this.board[row,col] =
            (this.status == GameStatus.WhiteIsNext) ?
                Stone.White : Stone.Black;

        // evaluate last stone
        if (this.IsGameOver (row, col, this.board[row,col]))
        {
            this.status = (this.status == GameStatus.WhiteIsNext) ?
                GameStatus.WhiteHasWon : GameStatus.BlackHasWon;

            if (this.GameStatusChanged != null)
            {
                GameStatusChangedEventArgs ea =
                    new GameStatusChangedEventArgs (this.status);
                this.GameStatusChanged (this, ea);
            }
        }
        else
        {
            this.status = (this.status == GameStatus.WhiteIsNext) ?
                GameStatus.BlackIsNext : GameStatus.WhiteIsNext;

            if (this.GameStatusChanged != null)
            {
                GameStatusChangedEventArgs ea =
                    new GameStatusChangedEventArgs (this.status);
                this.GameStatusChanged (this, ea);
            }
        }

        // initiate an 'OnPaint' method invocation
        this.Invalidate ();
    }

    private bool IsValidClick (int row, int col) 
    {
        // check values
        if (row < 0 || row >= MaxRow) 
            return false;
        if (col < 0 || col >= MaxCol) 
            return false;

        // is position occupied
        if (this.board[row, col] != Stone.Empty)
            return false;

        // is position valid
        for (int i = 0; i < row; i ++)
            if (this.board[i, col] == Stone.Empty) 
                return false;

        // game is already over
        if (this.status == GameStatus.WhiteHasWon ||
            this.status == GameStatus.BlackHasWon)
                return false;
 
        // position is okay
        return true;
    }

    private void DrawBoard (Graphics g) 
    {
        for (int i = 0; i < MaxRow; i ++) 
        {
            for (int j = 0; j < MaxCol; j ++) 
            {
                Stone s = this.board[MaxRow - i - 1, j];
                Brush b =
                    (s == Stone.White) ? Brushes.White :
                    (s == Stone.Black) ? Brushes.Black : Brushes.LightBlue;
                g.FillEllipse (
                    b, 10 + 5+ j * Diameter, 10 + 5 + i * Diameter, 40, 40
                );
            }
        }
    }

    public void ResetGame ()
    {
        this.status = GameStatus.WhiteIsNext;
        this.ResetBoard ();
        this.Invalidate ();
    }

    private void ResetBoard ()
    {
        // reset board
        for (int i = 0; i < MaxRow; i ++) 
            for (int j = 0; j < MaxCol; j ++) 
                this.board[i, j] = Stone.Empty;
    }
    
    private bool IsGameOver (int row, int col, Stone lastStone)
    {
        // handling vertical diagonal
        int count = 0;
        for (int i = 0; i < MaxRow; i ++)
        {
            if (this.board[i, col] == lastStone)
            {
                count ++;
                if (count == 4)
                    return true;
            }
            else
                count = 0;
        }

        // handling horizontal diagonal
        count = 0;
        for (int j = 0; j < MaxCol; j ++)
        {
            if (this.board[row, j] == lastStone)
            {
                count ++;
                if (count == 4)
                    return true;
            }
            else
                count = 0;
        }

        // handling first diagonal ([left,bottom] -> [right,top])
        int delta1 = Math.Min (row, col);
        int delta2 = Math.Min (MaxRow - row, MaxCol - col);

        int row1 = row - delta1;
        int col1 = col - delta1;
        int row2 = row + delta2;
        int col2 = col + delta2;

        count = 0;
        for (int i = row1, j = col1; i < row2 && j < col2; i ++, j ++)
        {
            if (this.board[i, j] == lastStone)
            {
                count ++;
                if (count == 4)
                    return true;
            }
            else
                count = 0;
        }

        // handling second diagonal ([left,top] -> [right,bottom])
        delta1 = Math.Min (MaxRow - row - 1, col);
        delta2 = Math.Min (row, MaxCol - col - 1);

        row1 = row + delta1;
        col1 = col - delta1;
        row2 = row - delta2;
        col2 = col + delta2;

        count = 0;
        for (int i = row1, j = col1; i >= row2 && j <= col2; i --, j ++)
        {
            if (this.board[i, j] == lastStone)
            {
                count ++;
                if (count == 4)
                    return true;
            }
            else
                count = 0;
        }

        return false;
    }
}
