using System;
using System.Windows.Forms;
using System.Drawing;

// class to encapsulate data for a 'ValveValue' event
class ValveValueEventArgs : EventArgs
{
    private int value;

    public ValveValueEventArgs (int value)
    {
        this.value = value;
    }

    public int Value
    {
        get { return this.value; }
    }
}

// delegate declaration for 'ValveValue' event
delegate void ValveValueEventHandler (Object sender, ValveValueEventArgs e);

// interface valve model
interface IValveModel
{
    // properties
    int Minimum { get; }
    int Maximum { get; }

    // methods
    void WidenOpening ();
    void NarrowOpening ();

    // event(s)
    event ValveValueEventHandler ValveValue;
}

// implementation of model
class ValveModelImpl : IValveModel
{
    // "openness" of the valve is limited (range 0 to 100)
    private const int minimum = 0;
    private const int maximum = 100;

    // valve state
    private int value;

    public event ValveValueEventHandler ValveValue;

    public ValveModelImpl ()
    {
        this.value = this.Minimum;
        this.ValveValue = null;
    }

    public int Minimum
    {
        get { return ValveModelImpl.minimum; }
    }

    public int Maximum
    {
        get { return ValveModelImpl.maximum; }
    }

    public void WidenOpening () 
    {
        if (this.value < this.Maximum) 
        {
            this.value ++;
            this.OnValveValue ();
        }
    } 

    public void NarrowOpening () 
    {
        if (this.value > this.Minimum) 
        {
            this.value --;
            this.OnValveValue ();
        }
    }

    protected virtual void OnValveValue ()
    {
        if (this.ValveValue != null)
        {
            ValveValueEventArgs e = new ValveValueEventArgs (this.value);
            this.ValveValue (this, e);
        }
    }
}

// implementation of a view: specialized label control
class ValveViewLabel : Label
{
    private IValveModel model;

    public ValveViewLabel (IValveModel model)
    {
        this.model = model;

        // initialize view
        this.BorderStyle = BorderStyle.Fixed3D;
        this.TextAlign = ContentAlignment.MiddleCenter;
        this.Text = this.model.Minimum.ToString ();

        // connect view with model
        this.model.ValveValue +=
            new ValveValueEventHandler (this.ValveValueChanged);
    }

    private void ValveValueChanged (Object sender, ValveValueEventArgs e)
    {
        this.Text = e.Value.ToString ();
    }
}

class ValveViewProgressBar : UserControl
{
    private IValveModel model;

    // class 'ProgressBar' is sealed, therefore using 'containment'
    private ProgressBar bar;

    public ValveViewProgressBar (IValveModel model)
    {
        this.model = model;

        // initialize progress bar control
        this.bar = new ProgressBar ();
        this.bar.Minimum = this.model.Minimum;
        this.bar.Maximum = this.model.Maximum;
        this.bar.Value = this.model.Minimum;
        this.bar.Step = 1;
        this.bar.Dock = DockStyle.Fill;
        this.Controls.Add (this.bar);

        // connect view with model
        this.model.ValveValue +=
            new ValveValueEventHandler (this.ValveValueChanged);
    }

    private void ValveValueChanged (Object sender, ValveValueEventArgs e)
    {
        this.bar.Value = e.Value;
    }
}

// implementation of two controllers: specialized button control
class OpenValveController : Button
{
    private IValveModel model;

    public OpenValveController (IValveModel model)
    {
        this.model = model;
        this.Text = "Open";
    }

    protected override void OnClick (EventArgs e)
    {
        base.OnClick (e);
        this.model.WidenOpening ();
    }
}

class CloseValveController : Button
{
    private IValveModel model;

    public CloseValveController (IValveModel model)
    {
        this.model = model;
        this.Text = "Close";
    }

    protected override void OnClick (EventArgs e)
    {
        base.OnClick (e);
        this.model.NarrowOpening ();
    }
}

// main window
class ValveForm : Form
{
    public ValveForm ()
    {
        // setup form
        this.ClientSize = new Size (500, 110);
        this.Text = "Model View Controller Paradigma: Valve Demo";

        // create model
        IValveModel model = new ValveModelImpl ();

        // create view (being connected to the model)
        ValveViewLabel lView = new ValveViewLabel (model);
        lView.Location = new Point (10, 10);
        lView.Width = this.ClientSize.Width - 20;
        lView.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;

        // create second view (being connected to the model)
        ValveViewProgressBar pbView = new ValveViewProgressBar (model);
        pbView.Location = new Point (10, 40);
        pbView.Width = this.ClientSize.Width - 20;
        pbView.Height = lView.Height;
        pbView.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;

        // create controller (being connected to the model)
        OpenValveController openCtrl = new OpenValveController (model);
        openCtrl.Location = new Point (10, 70);
        openCtrl.Width = 80;

        // create second controller (being connected to the model)
        CloseValveController closeCtrl = new CloseValveController (model);
        closeCtrl.Location = new Point (this.ClientSize.Width - 90, 70);
        closeCtrl.Width = 80;
        closeCtrl.Anchor = AnchorStyles.Top | AnchorStyles.Right;

        // add controls to the container
        this.Controls.Add (lView);
        this.Controls.Add (pbView);
        this.Controls.Add (openCtrl);
        this.Controls.Add (closeCtrl);
    }
}
