#define VERSION_01

#if VERSION_07
#define VERSION_04
#endif

#if VERSION_04a
#define VERSION_04
#endif

using System;
using System.Drawing;
using System.Windows.Forms;

public enum TrafficLightStatus
{
    Red,
    RedYellow,
    Yellow,
    Green
};

#if VERSION_01
public class TrafficLightControl : Control
{
    public TrafficLightControl ()
    {
        this.BackColor = Color.DarkGray;
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (100, 284);
        }       
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // draw surrounding border
        g.DrawRectangle (Pens.DarkBlue, 2, 2, 95, 279);
        g.DrawRectangle (Pens.DarkBlue, 4, 4, 91, 275);

        // draw lights
        g.FillEllipse (Brushes.Red, 10, 12, 80, 80);
        g.FillEllipse (Brushes.Yellow, 10, 102, 80, 80);
        g.FillEllipse (Brushes.Green, 10, 192, 80, 80);
    }
}
#endif

#if VERSION_02
public class TrafficLightControl : Control
{
    // status indicates current phase of the traffic light
    private TrafficLightStatus status;

    public TrafficLightControl ()
    {
        this.status = this.DefaultStatus;
        this.BackColor = Color.DarkGray;
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (100, 284);
        }
    }

    public TrafficLightStatus DefaultStatus
    {
        get
        {
            return TrafficLightStatus.Red;
        }
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.status;
        }

        set
        {
            if (this.status != value)
            {
                // update status
                this.status = value;

                // force a redraw of the control
                this.Invalidate ();
            }
        }
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // draw surrounding border
        g.DrawRectangle (Pens.DarkBlue, 2, 2, 95, 279);
        g.DrawRectangle (Pens.DarkBlue, 4, 4, 91, 275);

        // setup brushes
        Brush b1 = Brushes.DimGray;
        Brush b2 = b1;
        Brush b3 = b1;

        switch (this.status)
        {
            case TrafficLightStatus.Red:
                b1 = Brushes.Red;
                break;

            case TrafficLightStatus.RedYellow:
                b1 = Brushes.Red;
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Yellow:
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Green:
                b3 = Brushes.Green;
                break;
        }
        
        // draw lights
        g.FillEllipse (b1, 10, 12, 80, 80);
        g.FillEllipse (b2, 10, 102, 80, 80);
        g.FillEllipse (b3, 10, 192, 80, 80);
    }
}

#endif

#if VERSION_03
public class TrafficLightControl : Control
{
    // status indicates current phase of the traffic light
    private TrafficLightStatus status;

    // used to animate status shift
    private Timer timer;

    public TrafficLightControl ()
    {
        this.status = this.DefaultStatus;
        this.BackColor = Color.DarkGray;

        this.timer = new Timer ();
        this.timer.Interval = 1000;
        this.timer.Enabled = false;
        this.timer.Tick += new EventHandler (this.TimerEventHandler);
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (100, 284);
        }
    }

    public TrafficLightStatus DefaultStatus
    {
        get
        {
            return TrafficLightStatus.Red;
        }
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.StatusInternal;
        }

        set
        {
            if (this.timer.Enabled)
                return;

            // redirect to internal status attribute
            this.StatusInternal = value;
        }
    }

    private TrafficLightStatus StatusInternal
    {
        get
        {
            return this.status;
        }

        set
        {
            if (this.status != value)
            {
                // update status
                this.status = value;

                // force a redraw of the control
                this.Invalidate ();
            }
        }
    }

    public void SwitchToRed ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Green)
            return;
                    
        this.timer.Enabled = true;
    }
            
    public void SwitchToGreen ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Red)
            return;
                    
        this.timer.Enabled = true;
    }

    public void TimerEventHandler (Object sender, EventArgs e)
    {
        switch (this.StatusInternal)
        {
            case TrafficLightStatus.Red:
                this.StatusInternal = TrafficLightStatus.RedYellow;
                break;
            case TrafficLightStatus.RedYellow:
                this.StatusInternal = TrafficLightStatus.Green;
                this.timer.Enabled = false;
                break;
            case TrafficLightStatus.Yellow:
                this.StatusInternal = TrafficLightStatus.Red;
                this.timer.Enabled = false;
                break;
            case TrafficLightStatus.Green:
                this.StatusInternal = TrafficLightStatus.Yellow;
                break;
        }
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // draw surrounding border
        g.DrawRectangle (Pens.DarkBlue, 2, 2, 95, 279);
        g.DrawRectangle (Pens.DarkBlue, 4, 4, 91, 275);

        // setup brushes
        Brush b1 = Brushes.DimGray;
        Brush b2 = b1;
        Brush b3 = b1;

        switch (this.status)
        {
            case TrafficLightStatus.Red:
                b1 = Brushes.Red;
                break;

            case TrafficLightStatus.RedYellow:
                b1 = Brushes.Red;
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Yellow:
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Green:
                b3 = Brushes.Green;
                break;
        }
        
        // draw lights
        g.FillEllipse (b1, 10, 12, 80, 80);
        g.FillEllipse (b2, 10, 102, 80, 80);
        g.FillEllipse (b3, 10, 192, 80, 80);
    }
}
#endif

#if VERSION_04
public delegate void StatusChangedEventHandler (
    Object sender, StatusChangedEventArgs args
);

public class StatusChangedEventArgs : EventArgs
{
    private TrafficLightStatus status;

    public StatusChangedEventArgs (TrafficLightStatus status)
    {
        this.status = status;
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.status;
        }
    }
}

public class TrafficLightControl : Control
{
    // status indicates current phase of the traffic light
    private TrafficLightStatus status;

    // used to animate status shift
    private Timer timer;

    // event(s)
    public event StatusChangedEventHandler StatusChanged;

    public TrafficLightControl ()
    {
        this.status = this.DefaultStatus;
        this.BackColor = Color.DarkGray;

        this.timer = new Timer ();
        this.timer.Interval = 1000;
        this.timer.Enabled = false;
        this.timer.Tick += new EventHandler (this.TimerEventHandler);

        this.StatusChanged = null;
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (100, 284);
        }
    }

    public TrafficLightStatus DefaultStatus
    {
        get
        {
            return TrafficLightStatus.Red;
        }
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.StatusInternal;
        }

        set
        {
            if (this.timer.Enabled)
                return;

            // redirect to internal status attribute
            this.StatusInternal = value;
        }
    }

    private TrafficLightStatus StatusInternal
    {
        get
        {
            return this.status;
        }

        set
        {
            if (this.status != value)
            {
                // update status
                this.status = value;

                // inform subscribed clients about new status
                StatusChangedEventArgs args =
                    new StatusChangedEventArgs (this.status);
                this.OnStatusChanged (args);

                // force a redraw of the control
                this.Invalidate ();
            }
        }
    }

    protected virtual void OnStatusChanged (StatusChangedEventArgs args)
    {
        if (this.StatusChanged != null) 
        {
            // invoke registered clients
            this.StatusChanged (this, args);
        }
    }

    public void SwitchToRed ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Green)
            return;
                
        this.timer.Enabled = true;
    }
        
    public void SwitchToGreen ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Red)
            return;
                
        this.timer.Enabled = true;
    }

    private void TimerEventHandler (Object sender, EventArgs e)
    {
        switch (this.StatusInternal)
        {
            case TrafficLightStatus.Red:
                this.StatusInternal = TrafficLightStatus.RedYellow;
                break;
            case TrafficLightStatus.RedYellow:
                this.StatusInternal = TrafficLightStatus.Green;
                this.timer.Enabled = false;
                break;
            case TrafficLightStatus.Yellow:
                this.StatusInternal = TrafficLightStatus.Red;
                this.timer.Enabled = false;
                break;
            case TrafficLightStatus.Green:
                this.StatusInternal = TrafficLightStatus.Yellow;
                break;
        }
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // draw surrounding border
        g.DrawRectangle (Pens.DarkBlue, 2, 2, 95, 279);
        g.DrawRectangle (Pens.DarkBlue, 4, 4, 91, 275);

        // setup brushes
        Brush b1 = Brushes.DimGray;
        Brush b2 = b1;
        Brush b3 = b1;

        switch (this.status)
        {
            case TrafficLightStatus.Red:
                b1 = Brushes.Red;
                break;

            case TrafficLightStatus.RedYellow:
                b1 = Brushes.Red;
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Yellow:
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Green:
                b3 = Brushes.Green;
                break;
        }
        
        // draw lights
        g.FillEllipse (b1, 10, 12, 80, 80);
        g.FillEllipse (b2, 10, 102, 80, 80);
        g.FillEllipse (b3, 10, 192, 80, 80);
    }
}
#endif

#if VERSION_05
public delegate void StatusChangedEventHandler (Object sender, StatusChangedEventArgs args);
public delegate void RedRequestEventHandler (Object sender, EventArgs e);
public delegate void RedAcknowledgeEventHandler (Object sender, EventArgs e);

public class StatusChangedEventArgs : EventArgs
{
    private TrafficLightStatus status;

    public StatusChangedEventArgs (TrafficLightStatus status)
    {
        this.status = status;
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.status;
        }
    }
}

public class TrafficLightControl : Control
{
    // status indicates current phase of the traffic light
    private TrafficLightStatus status;

    // used to animate status shift
    private Timer timer;

    private enum ShiftStates
    {
        Idle,                        // idle

        Sending_RedRequest,          // sending 'RedRequest' event to subscribers
        Waiting_For_RedAcknowledge,  // waiting for 'RedAcknowledge' event
        Red_To_RedYellow,            // moving from 'Red' -> 'Red-Yellow'
        RedYellow_To_Green,          // moving from 'Red-Yellow' -> 'Green',
        // stop timer

        Green_To_Yellow,             // moving from 'Green' -> 'Yellow'
        Yellow_To_Red,               // moving from 'Yellow' -> 'Red'
        Sending_RedAcknowledge,      // sending 'RedAcknowledge' event to subscribers,
        // stop timer 
    };
    private ShiftStates shiftState;

    // events
    public event StatusChangedEventHandler StatusChanged;
    public event RedRequestEventHandler RedRequest;
    public event RedAcknowledgeEventHandler RedAcknowledge;

    public TrafficLightControl ()
    {
        this.status = this.DefaultStatus;
        this.BackColor = Color.DarkGray;

        this.timer = new Timer ();
        this.timer.Interval = 1500;
        this.timer.Enabled = false;
        this.timer.Tick += new EventHandler (this.TimerEventHandler);

        this.shiftState = ShiftStates.Idle;

        this.StatusChanged = null;
        this.RedRequest = null;
        this.RedAcknowledge = null;
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (100, 284);
        }
    }

    public TrafficLightStatus DefaultStatus
    {
        get
        {
            return TrafficLightStatus.Red;
        }
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.StatusInternal;
        }

        set
        {
            if (this.timer.Enabled)
                return;

            // redirect to internal status attribute
            this.StatusInternal = value;
        }
    }

    private TrafficLightStatus StatusInternal
    {
        get
        {
            return this.status;
        }

        set
        {
            if (this.status != value)
            {
                // update status
                this.status = value;

                // inform subscribed clients about new status
                StatusChangedEventArgs args =
                    new StatusChangedEventArgs (this.status);
                this.OnStatusChanged (args);

                // force a redraw of the control
                this.Invalidate ();
            }
        }
    }

    protected virtual void OnStatusChanged (StatusChangedEventArgs args)
    {
        if (this.StatusChanged != null) 
        {
            // invoke registered clients
            this.StatusChanged (this, args);
        }
    }

    public void SwitchToRed ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Green)
            return;

        this.shiftState = ShiftStates.Green_To_Yellow;
        this.timer.Enabled = true;
    }
            
    public void SwitchToGreen ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Red)
            return;

        this.shiftState = ShiftStates.Sending_RedRequest;
        this.timer.Enabled = true;
    }

    private void TimerEventHandler (Object sender, EventArgs e)
    {
        Console.WriteLine ("Timer:  {0}: {1}", this.Name, this.shiftState.ToString());
            
        switch (this.shiftState)
        {
            case ShiftStates.Sending_RedRequest:
                // wait for connected traffic lights, if any
                if (this.RedRequest != null)
                {
                    // notifying 'RedRequest' event subscribers
                    this.RedRequest (this, EventArgs.Empty);
                    this.shiftState = ShiftStates.Waiting_For_RedAcknowledge;
                }
                else
                {
                    this.shiftState = ShiftStates.Red_To_RedYellow;
                }
                break;

            case ShiftStates.Waiting_For_RedAcknowledge:
                // waiting for acknowledge
                this.shiftState = ShiftStates.Waiting_For_RedAcknowledge;
                break;

            case ShiftStates.Red_To_RedYellow:
                // moving from 'Red' -> 'Red-Yellow'
                this.StatusInternal = TrafficLightStatus.RedYellow;
                this.shiftState = ShiftStates.RedYellow_To_Green;
                break;

            case ShiftStates.RedYellow_To_Green:
                // moving from 'Red-Yellow' -> 'Green'
                this.StatusInternal = TrafficLightStatus.Green;

                // stop timer and reset shift state
                this.timer.Enabled = false;
                this.shiftState = ShiftStates.Idle;
                break;

            case ShiftStates.Green_To_Yellow:
                // moving from 'Green' -> 'Yellow'
                this.StatusInternal = TrafficLightStatus.Yellow;
                this.shiftState = ShiftStates.Yellow_To_Red;
                break;

            case ShiftStates.Yellow_To_Red:
                // moving from 'Yellow' -> 'Red'
                this.StatusInternal = TrafficLightStatus.Red;
                this.shiftState = ShiftStates.Sending_RedAcknowledge;
                break;

            case ShiftStates.Sending_RedAcknowledge:
                // notifying 'RedAcknowledge' event subscribers
                if (this.RedAcknowledge != null)
                    this.RedAcknowledge (this, EventArgs.Empty);

                // stop timer and reset shift state
                this.timer.Enabled = false;
                this.shiftState = ShiftStates.Idle;
                break;
        }
    }

    public void ResumeSwitchToGreen ()
    {
        // resume phase shift
        Console.WriteLine ("in ResumeSwitchToGreen");
        this.shiftState = ShiftStates.Red_To_RedYellow;
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // draw surrounding border
        g.DrawRectangle (Pens.DarkBlue, 2, 2, 95, 279);
        g.DrawRectangle (Pens.DarkBlue, 4, 4, 91, 275);

        // setup brushes
        Brush b1 = Brushes.DimGray;
        Brush b2 = b1;
        Brush b3 = b1;

        switch (this.status)
        {
            case TrafficLightStatus.Red:
                b1 = Brushes.Red;
                break;

            case TrafficLightStatus.RedYellow:
                b1 = Brushes.Red;
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Yellow:
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Green:
                b3 = Brushes.Green;
                break;
        }
        
        // draw lights
        g.FillEllipse (b1, 10, 12, 80, 80);
        g.FillEllipse (b2, 10, 102, 80, 80);
        g.FillEllipse (b3, 10, 192, 80, 80);
    }
}
#endif

#if VERSION_06
public delegate void StatusChangedEventHandler (Object sender, StatusChangedEventArgs args);
public delegate void RedRequestEventHandler (Object sender, EventArgs e);
public delegate void RedAcknowledgeEventHandler (Object sender, EventArgs e);

public class StatusChangedEventArgs : EventArgs
{
    private TrafficLightStatus status;

    public StatusChangedEventArgs (TrafficLightStatus status)
    {
        this.status = status;
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.status;
        }
    }
}

public class RedRequestAdapter
{
    TrafficLightControl target;

    public RedRequestAdapter (
        TrafficLightControl source, TrafficLightControl target)
    {
        this.target = target;
        source.RedRequest +=
            new RedRequestEventHandler (this.GotRedRequest);
    }

    public void GotRedRequest (Object sender, EventArgs e)
    {
        this.target.SwitchToRed ();
    }
}

public class RedAcknowledgeAdapter
{
    TrafficLightControl target;

    public RedAcknowledgeAdapter (
        TrafficLightControl source, TrafficLightControl target)
    {
        this.target = target;
        source.RedAcknowledge +=
            new RedAcknowledgeEventHandler (this.GotRedAcknowledge);
    }

    public void GotRedAcknowledge (Object sender, EventArgs e)
    {
        this.target.ResumeSwitchToGreen ();
    }
}
    
public class TrafficLightControl : Control
{
    // status indicates current phase of the traffic light
    private TrafficLightStatus status;

    // used to animate status shift
    private Timer timer;

    private enum ShiftStates
    {
        Idle,                        // idle

        Sending_RedRequest,          // sending 'RedRequest' event to subscribers
        Waiting_For_RedAcknowledge,  // waiting for 'RedAcknowledge' event
        Red_To_RedYellow,            // moving from 'Red' -> 'Red-Yellow'
        RedYellow_To_Green,          // moving from 'Red-Yellow' -> 'Green',
        // stop timer

        Green_To_Yellow,             // moving from 'Green' -> 'Yellow'
        Yellow_To_Red,               // moving from 'Yellow' -> 'Red'
        Sending_RedAcknowledge,      // sending 'RedAcknowledge' event to subscribers,
        // stop timer 
    };
    private ShiftStates shiftState;

    // events
    public event StatusChangedEventHandler StatusChanged;
    public event RedRequestEventHandler RedRequest;
    public event RedAcknowledgeEventHandler RedAcknowledge;

    public TrafficLightControl ()
    {
        this.status = this.DefaultStatus;
        this.BackColor = Color.DarkGray;

        this.timer = new Timer ();
        this.timer.Interval = 1500;
        this.timer.Enabled = false;
        this.timer.Tick += new EventHandler (this.TimerEventHandler);

        this.shiftState = ShiftStates.Idle;

        this.StatusChanged = null;
        this.RedRequest = null;
        this.RedAcknowledge = null;
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (100, 284);
        }
    }

    public TrafficLightStatus DefaultStatus
    {
        get
        {
            return TrafficLightStatus.Red;
        }
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.StatusInternal;
        }

        set
        {
            if (this.timer.Enabled)
                return;

            // redirect to internal status attribute
            this.StatusInternal = value;
        }
    }

    private TrafficLightStatus StatusInternal
    {
        get
        {
            return this.status;
        }

        set
        {
            if (this.status != value)
            {
                // update status
                this.status = value;

                // inform subscribed clients about new status
                StatusChangedEventArgs args =
                    new StatusChangedEventArgs (this.status);
                this.OnStatusChanged (args);

                // force a redraw of the control
                this.Invalidate ();
            }
        }
    }

    protected virtual void OnStatusChanged (StatusChangedEventArgs args)
    {
        if (this.StatusChanged != null) 
        {
            // invoke registered clients
            this.StatusChanged (this, args);
        }
    }

    public void SwitchToRed ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Green)
            return;

        this.shiftState = ShiftStates.Green_To_Yellow;
        this.timer.Enabled = true;
    }
            
    public void SwitchToGreen ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Red)
            return;

        this.shiftState = ShiftStates.Sending_RedRequest;
        this.timer.Enabled = true;
    }

    private void TimerEventHandler (Object sender, EventArgs e)
    {
        Console.WriteLine ("Timer:  {0}: {1}", this.Name, this.shiftState.ToString());
            
        switch (this.shiftState)
        {
            case ShiftStates.Sending_RedRequest:
                // wait for connected traffic lights, if any
                if (this.RedRequest != null)
                {
                    // notifying 'RedRequest' event subscribers
                    this.RedRequest (this, EventArgs.Empty);
                    this.shiftState = ShiftStates.Waiting_For_RedAcknowledge;
                }
                else
                {
                    this.shiftState = ShiftStates.Red_To_RedYellow;
                }
                break;

            case ShiftStates.Waiting_For_RedAcknowledge:
                // waiting for acknowledge
                this.shiftState = ShiftStates.Waiting_For_RedAcknowledge;
                break;

            case ShiftStates.Red_To_RedYellow:
                // moving from 'Red' -> 'Red-Yellow'
                this.StatusInternal = TrafficLightStatus.RedYellow;
                this.shiftState = ShiftStates.RedYellow_To_Green;
                break;

            case ShiftStates.RedYellow_To_Green:
                // moving from 'Red-Yellow' -> 'Green'
                this.StatusInternal = TrafficLightStatus.Green;

                // stop timer and reset shift state
                this.timer.Enabled = false;
                this.shiftState = ShiftStates.Idle;
                break;

            case ShiftStates.Green_To_Yellow:
                // moving from 'Green' -> 'Yellow'
                this.StatusInternal = TrafficLightStatus.Yellow;
                this.shiftState = ShiftStates.Yellow_To_Red;
                break;

            case ShiftStates.Yellow_To_Red:
                // moving from 'Yellow' -> 'Red'
                this.StatusInternal = TrafficLightStatus.Red;
                this.shiftState = ShiftStates.Sending_RedAcknowledge;
                break;

            case ShiftStates.Sending_RedAcknowledge:
                // notifying 'RedAcknowledge' event subscribers
                if (this.RedAcknowledge != null)
                    this.RedAcknowledge (this, EventArgs.Empty);

                // stop timer and reset shift state
                this.timer.Enabled = false;
                this.shiftState = ShiftStates.Idle;
                break;
        }
    }

    public void ResumeSwitchToGreen ()
    {
        // resume phase shift
        Console.WriteLine ("in ResumeSwitchToGreen");
        this.shiftState = ShiftStates.Red_To_RedYellow;
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // draw surrounding border
        g.DrawRectangle (Pens.DarkBlue, 2, 2, 95, 279);
        g.DrawRectangle (Pens.DarkBlue, 4, 4, 91, 275);

        // setup brushes
        Brush b1 = Brushes.DimGray;
        Brush b2 = b1;
        Brush b3 = b1;

        switch (this.status)
        {
            case TrafficLightStatus.Red:
                b1 = Brushes.Red;
                break;

            case TrafficLightStatus.RedYellow:
                b1 = Brushes.Red;
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Yellow:
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Green:
                b3 = Brushes.Green;
                break;
        }
        
        // draw lights
        g.FillEllipse (b1, 10, 12, 80, 80);
        g.FillEllipse (b2, 10, 102, 80, 80);
        g.FillEllipse (b3, 10, 192, 80, 80);
    }
}
#endif

#if VERSION_07
public class TrafficLight : UserControl
{
    private TrafficLightControl tl;
    private Button b;
    private Timer timer;
    private int duration;

    public TrafficLight ()
    {
        // setup user control
        this.Size = new Size (120, 335);
        this.BackColor = Color.DarkGray;

        // setup traffic light control
        this.tl = new TrafficLightControl ();
        this.tl.Location = new Point (10, 10);
        this.Controls.Add (this.tl);

        // setup button
        this.b = new Button ();
        this.b.Text = "Press";
        this.b.Location = new Point (10, 300);
        this.b.Size = new Size (100, 23);
        this.b.Click += new EventHandler (this.ClickHandler);
        this.Controls.Add (this.b);

        // setup timer control
        this.timer = new Timer ();
        this.timer.Interval = 1000;
        this.timer.Tick += new EventHandler (this.TimerEventHandler);
        this.duration = 0;
        this.timer.Enabled = true;
    }

    private void ClickHandler (Object sender, EventArgs e)
    {
        this.tl.SwitchToGreen ();
    }

    private void TimerEventHandler (Object sender, EventArgs e)
    {
        if (this.tl.Status == TrafficLightStatus.Green)
        {
            this.duration ++;
            if (this.duration == 5)
            {
                this.tl.SwitchToRed ();
                this.duration = 0;
            }
        }
    }
}
#endif

#if VERSION_MVC
public delegate void StatusChangedEventHandler (
    Object sender, StatusChangedEventArgs args
);

public class StatusChangedEventArgs : EventArgs
{
    private TrafficLightStatus status;

    public StatusChangedEventArgs (TrafficLightStatus status)
    {
        this.status = status;
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.status;
        }
    }
}

// interface traffic light model
public interface ITrafficLightModel
{
    // properties
    TrafficLightStatus Status { get; }

    // methods
    void SwitchToRed ();
    void SwitchToGreen ();

    // events
    event StatusChangedEventHandler StatusChanged;
}

// implementation of model 'ITrafficLightModel'
public class TrafficLightModelImpl : ITrafficLightModel
{
    // status indicates current phase of the traffic light
    private TrafficLightStatus status;

    // used to animate status shift
    private Timer timer;

    // events
    public event StatusChangedEventHandler StatusChanged;

    public TrafficLightModelImpl ()
    {
        this.status = TrafficLightStatus.Red;

        this.timer = new Timer ();
        this.timer.Interval = 1000;
        this.timer.Enabled = false;
        this.timer.Tick += new EventHandler (this.TimerEventHandler);
        this.StatusChanged = null;
    }

    public TrafficLightStatus Status
    {
        get
        {
            return this.StatusInternal;
        }

        set
        {
            if (this.timer.Enabled)
                return;

            // redirect to internal status attribute
            this.StatusInternal = value;
        }
    }

    private TrafficLightStatus StatusInternal
    {
        get
        {
            return this.status;
        }

        set
        {
            if (this.status != value)
            {
                // update status
                this.status = value;

                // inform subscribed clients about new status
                StatusChangedEventArgs args =
                    new StatusChangedEventArgs (this.status);
                this.OnStatusChanged (args);
            }
        }
    }

    protected virtual void OnStatusChanged (StatusChangedEventArgs args)
    {
        if (this.StatusChanged != null) 
        {
            // invoke registered clients
            this.StatusChanged (this, args);
        }
    }

    public void SwitchToRed ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Green)
            return;
                
        this.timer.Enabled = true;
    }
        
    public void SwitchToGreen ()
    {
        if (this.timer.Enabled)
            return;

        if (this.Status != TrafficLightStatus.Red)
            return;
                
        this.timer.Enabled = true;
    }

    private void TimerEventHandler (Object sender, EventArgs e)
    {
        switch (this.StatusInternal)
        {
            case TrafficLightStatus.Red:
                this.StatusInternal = TrafficLightStatus.RedYellow;
                break;
            case TrafficLightStatus.RedYellow:
                this.StatusInternal = TrafficLightStatus.Green;
                this.timer.Enabled = false;
                break;
            case TrafficLightStatus.Yellow:
                this.StatusInternal = TrafficLightStatus.Red;
                this.timer.Enabled = false;
                break;
            case TrafficLightStatus.Green:
                this.StatusInternal = TrafficLightStatus.Yellow;
                break;
        }
    }
}

// implementation of a view: user defined control
public class TrafficLightView : Control
{
    private ITrafficLightModel model;
    private TrafficLightStatus status;

    public TrafficLightView (ITrafficLightModel model)
    {
        this.model = model;
        this.status = this.model.Status;
        this.BackColor = Color.DarkGray;

        // connect view with model
        this.model.StatusChanged +=
            new StatusChangedEventHandler (this.StatusHasChanged);
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (100, 284);
        }
    }

    private void StatusHasChanged (Object sender, StatusChangedEventArgs e)
    {
        this.status = e.Status;

        // force a redraw of the control
        this.Invalidate ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // take care of registered delegates
        base.OnPaint (e);

        Graphics g = e.Graphics;

        // draw surrounding border
        g.DrawRectangle (Pens.DarkBlue, 2, 2, 95, 279);
        g.DrawRectangle (Pens.DarkBlue, 4, 4, 91, 275);

        // setup brushes
        Brush b1 = Brushes.DimGray;
        Brush b2 = b1;
        Brush b3 = b1;

        switch (this.status)
        {
            case TrafficLightStatus.Red:
                b1 = Brushes.Red;
                break;

            case TrafficLightStatus.RedYellow:
                b1 = Brushes.Red;
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Yellow:
                b2 = Brushes.Yellow;
                break;

            case TrafficLightStatus.Green:
                b3 = Brushes.Green;
                break;
        }
        
        // draw lights
        g.FillEllipse (b1, 10, 12, 80, 80);
        g.FillEllipse (b2, 10, 102, 80, 80);
        g.FillEllipse (b3, 10, 192, 80, 80);
    }
}

// implementation of a second view: specialized text box control
public class TextBoxView : TextBox
{
    private ITrafficLightModel model;

    public TextBoxView (ITrafficLightModel model)
    {
        this.model = model;

        // no response to user interaction
        this.Enabled = false;
        this.Text = model.Status.ToString ();
        this.Width = 160;

        // connect view with model
        this.model.StatusChanged +=
            new StatusChangedEventHandler (this.StatusHasChanged);
    }

    private void StatusHasChanged (Object sender, StatusChangedEventArgs e)
    {
        this.Text = e.Status.ToString ();
    }
}

// implementation of a controller: specialized button control
public class TrafficLightRedController : Button
{
    private ITrafficLightModel model;

    public TrafficLightRedController (ITrafficLightModel model)
    {
        this.model = model;
        this.Text = "Go Red";
        this.BackColor = Color.LightGray;
        this.Width = 160;
    }

    protected override void OnClick (EventArgs e)
    {
        // enable registered clients to receive 'Click' events
        base.OnClick (e);

        model.SwitchToRed ();
    }
}

public class TrafficLightGreenController : Button
{
    private ITrafficLightModel model;

    public TrafficLightGreenController (ITrafficLightModel model)
    {
        this.model = model;
        this.Text = "Go Green";
        this.BackColor = Color.LightGray;
        this.Width = 160;
    }

    protected override void OnClick (EventArgs e)
    {
        // enable registered clients to receive 'Click' events
        base.OnClick (e);

        model.SwitchToGreen ();
    }
}
#endif

