using System;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Text;

class BernsteinForm : Form
{
    private const int HEIGHT = 400;
    private const int WIDTH = 300;
    private const int RESOLUTION = WIDTH;

    private int[] polynoms;
    private int degree; // degree of bernstein polynomials

    // default c'tor
    public BernsteinForm (int[] polynoms, int degree)
    {
        // set caption bar text and size of this form
        this.Text = "Bernstein Polynomials";
        this.BackColor = Color.White;
        this.ClientSize = new Size (400, 450);

        // control must be redrawn when it is resized
        this.ResizeRedraw = true;

        this.degree = degree;
        this.polynoms = polynoms;
    }

    private static long [][] binomialCoefficients;

    static BernsteinForm ()
    {
        // create column array
        BernsteinForm.binomialCoefficients = new long [52][];

        // initialize first row
        long [] firstRow = new long [1];
        firstRow [0] = 1;
        BernsteinForm.binomialCoefficients [0] = firstRow;

        // initialize remaining rows
        for (int n = 1; n < binomialCoefficients.Length; n ++) 
        {
            long [] nextRow = new long [n + 1];

            nextRow [0] = 1;
            nextRow [n] = 1;

            for (int k = 1; k <= (n) / 2; k ++) 
            {
                nextRow [k] = (nextRow [k - 1] * (n - k + 1)) / k;
                nextRow [n - k] = nextRow [k];
            }
            BernsteinForm.binomialCoefficients [n] = nextRow;
        }    
    }

    private double computeBernsteinValueNonRecursive02 (double t, int i, int n) 
    {
        double r;
        if (i == 0) 
        {
            r = 1.0;
            for (int k = 0; k < n; k ++) 
            {
                r *= (1.0 - t);
            }
        }
        else if (n == i) 
        {
            r = 1.0;
            for (int k = 0; k < n; k ++) 
            {
                r *= t;
            }
        }
        else 
        {
            double r1 = 1.0;
            for (int k = 0; k < i; k ++) 
            {
                r1 *= t;
            }

            double r2 = 1.0;
            for (int k = 0; k < n - i; k ++) 
            {
                r2 *= (1.0 - t);
            }

            r = binomialCoefficients [n][i] * r1 * r2;
        }
        return r;
    }

    public void drawBernsteinPolynomial (Graphics g, int i, int n) 
    {
        Pen p = new Pen (Color.Black, 2);
        double x0 = 0.0;
        double y0 = computeBernsteinValueNonRecursive02 (x0, i, n);
        double resolution = RESOLUTION;

        int ix0 = (int) Math.Round (WIDTH * x0);
        int iy0 = (int) Math.Round (HEIGHT * y0);

        for (int k = 1; k <= resolution; k ++) 
        {
            double x1 = ((double) k) / ((double) resolution);
            double y1 = computeBernsteinValueNonRecursive02 (x1, i, n);

            int ix1 = (int) Math.Round (WIDTH * x1);
            int iy1 = (int) Math.Round (HEIGHT * y1);

            g.DrawLine (p, ix0, -iy0, ix1, -iy1);

            ix0 = ix1;
            iy0 = iy1;
        }
        p.Dispose ();
    }

    public void drawCoordinateSystem (Graphics g, int width, int height) 
    {
        Font f = new Font ("Courier New", 12);
        Brush b = new SolidBrush (Color.Black);

        g.DrawLine (Pens.Black, -10, 0, width + 20, 0);
        g.DrawLine (Pens.Black, 0, 10, 0, -height);

//        g.DrawString ("0", f, b, -15, -5);
//        g.DrawString ("0", f, b,  10, 20);
//
//        g.DrawString ("1", f, b, width - 10, 20);
//        g.DrawString ("1", f, b, -15, -height + 10);

        f.Dispose ();
        b.Dispose ();
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // move origin of current graphics context
        e.Graphics.TranslateTransform (50, HEIGHT + 20); 
 
        this.drawCoordinateSystem (e.Graphics, WIDTH, HEIGHT);

        if (this.polynoms != null) 
        {
            for (int i = 0; i < this.polynoms.Length; i ++)
                // just draw specific bernstein polynomials
                drawBernsteinPolynomial (e.Graphics, this.polynoms[i], degree);
        } 
        else 
        {
            // draw all of them
            for (int i = 0; i <= degree; i ++)
                drawBernsteinPolynomial (e.Graphics, i, degree);
        }
    }
}

class Demo
{
    public static void Main ()
    {
        int[] polynoms = { 0, 1 };
        int degree = 30;
        BernsteinForm f = new BernsteinForm (null, degree);
        Application.Run (f);
    }
}

