using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

class Point
{
    private int x, y;

    public Point (int x, int y) 
    {
        this.x = x;
        this.y = y;
    }

    public int X
    {
        get
        {
            return this.x;
        }
    }

    public int Y
    {
        get
        {
            return this.y;
        }
    }

    // compute the linear interpolation of two points
    public static Point Interpolate (Point p0, Point p1, double t)
    {
        double x = t * p1.x + (1 - t) * p0.x;
        double y = t * p1.y + (1 - t) * p0.y;
        Point p = new Point ((int) Math.Round (x), (int) Math.Round (y));
        return p;
    }

    // compute the distance between two points
    public static double Distance (Point p0, Point p1) 
    {
        int dx = p1.x - p0.x;
        int dy = p1.y - p0.y;
        int sum = dx * dx + dy * dy;
        return Math.Sqrt ((double) sum);
    }
}

public enum BezierMode { Create, Edit, Delete };

public class BezierPanel : Control
{
    private BezierMode mode;
    private ArrayList controlPoints;
    private int dragIndex;

    private bool showControlPoints;
    private bool showBezierCurve;
    private bool showConstruction;

    private int resolution;
    private double constructionPosition;

    private Font f;

    public BezierPanel ()
    {
        this.mode = BezierMode.Create;
        this.controlPoints = new ArrayList ();
        this.dragIndex = -1;

        this.showControlPoints = true;
        this.showBezierCurve = true;
        this.showConstruction = false;

        this.resolution = 50;

        this.constructionPosition = 0.5;

        this.BackColor = Color.White;

        this.f = new Font ("SansSerif", 8, FontStyle.Regular);

        // enable double-buffering
        this.SetStyle (
            ControlStyles.DoubleBuffer | 
            ControlStyles.UserPaint | 
            ControlStyles.AllPaintingInWmPaint,
            true
        );
    }

    public BezierMode Mode
    {
        get
        {
            return this.mode;
        }

        set
        {
            this.mode = value;
            this.Invalidate ();
        }
    }

    public int Resolution
    {
        get
        {
            return this.resolution;
        }

        set
        {
            if (value >= 1 && value <= 100)
            {
                this.resolution = value;
                this.Invalidate ();
            }
        }
    }

    public double ConstructionPosition
    {
        get
        {
            return this.constructionPosition;
        }

        set
        {
            if (value >= 0.0 && value <= 1.0)
            {
                this.constructionPosition = value;
                this.Invalidate ();
            }
        }
    }

    public bool ShowControlPoints
    {
        get
        {
            return this.showControlPoints;
        }

        set
        {
            this.showControlPoints = value;
            this.Invalidate ();
        }
    }

    public bool ShowBezierCurve
    {
        get
        {
            return this.showBezierCurve;
        }

        set
        {
            this.showBezierCurve = value;
            this.Invalidate ();
        }
    }

    public bool ShowConstruction
    {
        get
        {
            return this.showConstruction;
        }

        set
        {
            this.showConstruction = value;
            this.Invalidate ();
        }
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        base.OnPaint (e);

        if (this.showControlPoints)
            this.DrawControlPoints (e.Graphics);

        if (this.showBezierCurve)
            this.DrawBezierCurve (e.Graphics);

        if (this.showConstruction)
            this.DrawConstructionPoints (e.Graphics);
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        base.OnMouseDown (e);

        if (this.mode == BezierMode.Create)
        {
            // add new control point
            Point p = new Point (e.X, e.Y);
            this.controlPoints.Add (p);
            this.Invalidate ();
        }
        else if (this.mode == BezierMode.Edit)
        {
            Point p = new Point (e.X, e.Y);
            dragIndex = this.GetNearestPointIndex (p);
            if (dragIndex == -1) 
            {
                return;
            }

            // update control point
            this.controlPoints[dragIndex] = p;
            this.Invalidate ();
        }
        else if (this.mode == BezierMode.Delete)
        {
            Point p = new Point (e.X, e.Y);
            int index = this.GetNearestPointIndex (p);
            if (index == -1) 
            {
                return;
            }

            // update control points
            this.controlPoints.RemoveAt (index);
            this.Invalidate ();
        }
    }

    protected override void OnMouseUp (MouseEventArgs e)
    {
        base.OnMouseUp (e);

        if (this.mode == BezierMode.Edit) 
        {
            this.dragIndex = -1;
        }
    }

    protected override void OnMouseMove (MouseEventArgs e)
    {
        base.OnMouseMove (e);

        if (e.Button == MouseButtons.None)
        {
            // mouse pointer is moved, no button is pressed
        }
        else if (e.Button == MouseButtons.Left)
        {
            // mouse pointer is moved, left button is pressed
            if (this.mode == BezierMode.Edit)
            {
                Point p = new Point (e.X, e.Y);
                if (dragIndex != -1) 
                {

                    // update control point
                    this.controlPoints[dragIndex] = p;
                    this.Invalidate ();
                }
            }
        }
    }

    public void ClearControlPoints () 
    {
        this.controlPoints.Clear ();
        this.Invalidate ();
    }

    private void DrawControlPoints (Graphics g) 
    {
        int numPoints = this.controlPoints.Count;
        if (numPoints == 0)
            return;

        Point p0, p1;
        p0 = (Point) this.controlPoints[0];
        this.DrawControlPoint (g, p0.X, p0.Y, "0");

        for (int i = 1; i < numPoints; i ++) 
        {
            p1 = (Point) this.controlPoints[i];
            String s = i.ToString ();
            this.DrawControlPoint (g, p1.X, p1.Y, s);
            g.DrawLine (Pens.Blue, p0.X, p0.Y, p1.X, p1.Y);
            p0 = p1;
        }
    }

    private void DrawControlPoint (Graphics g, int x, int y, String text) 
    {
        g.DrawRectangle (Pens.Red, x - 5, y - 5, 10, 10);
        g.DrawString (text, f, Brushes.Blue, x + 8, y - 8);
    }

    private void DrawBezierCurve (Graphics g) 
    {
        int numPoints = this.controlPoints.Count;
        if (numPoints < 3)
            return;

        Point p0 = (Point) this.controlPoints[0];
        for (int i = 1; i <= this.resolution; i ++) {
            double t = ((double) i) / ((double) this.resolution);
            Point p1 = this.ComputeBezierPoint (t);
            g.DrawLine (Pens.Blue, p0.X, p0.Y, p1.X, p1.Y);
            p0 = p1;
        }
    }

    // helper function to compute a bezier point
    // defined by the control polygon at value t
    private Point ComputeBezierPoint (double t) 
    {
        int numPoints = this.controlPoints.Count;
        Point[] array = new Point[numPoints];
        this.controlPoints.CopyTo (array);

        for (int n = array.Length; n > 0; n --) 
            for (int i = 1; i < n; i ++) 
                array[i-1] = Point.Interpolate (array[i-1], array[i], t);

        return array[0];
    }

    private void DrawConstructionPoints (Graphics g) 
    {
        int numPoints = this.controlPoints.Count;
        if (numPoints < 3)
            return;

        Point[][] constructionPoints = this.ComputeConstructionPoints (this.constructionPosition);
        numPoints = constructionPoints.Length;

        for (int i = numPoints - 2; i >= 0; i--) 
        {
            Point[] array = constructionPoints[i];

            Point p0 = array[0];
            for (int j = 1; j <= i; j ++) 
            {
                Point p1 = array[j];
                g.DrawLine (Pens.Black, p0.X, p0.Y, p1.X, p1.Y);
                p0 = p1;
            }
        }

        // drawing Bezier point itself
        Point pBezier = constructionPoints[0][0];
        String pos = this.constructionPosition.ToString ();
        this.DrawControlPoint (g, pBezier.X, pBezier.Y, pos);
    }

    // helper function to compute all the construction points
    // needed to compute a single bezier point at value t
    private Point[][] ComputeConstructionPoints (double t)
    {
        int numPoints = controlPoints.Count;
        Point[][] constructionPoints = new Point[numPoints][];

        Point[] tmpArray = new Point[numPoints];
        controlPoints.CopyTo (tmpArray);
        constructionPoints[numPoints-1] = tmpArray;

        for (int n = numPoints - 1; n > 0; n --) 
        {
            tmpArray = new Point[n];
            for (int i = 1; i <= n; i ++) 
            {
                Point p0 = constructionPoints[n][i-1];
                Point p1 = constructionPoints[n][i];
                Point p = Point.Interpolate (p0, p1, t);
                tmpArray[i-1] = p;
            }
            constructionPoints[n - 1] = tmpArray;
        }
        return constructionPoints;
    }

    // helper function to find the nearest control point
    private int GetNearestPointIndex (Point p0) 
    {
        int numPoints = controlPoints.Count;
        if (numPoints == 0)
            return -1;

        int index = -1;
        double dist = Double.MaxValue;

        for (int i = 0; i < numPoints; i ++) 
        {
            Point p1 = (Point) this.controlPoints[i];
            double newDist = Point.Distance (p0, p1);
            if (newDist < dist) 
            {
                dist = newDist;
                index = i;
            }
        }
        return (dist < 10.0) ? index : -1;
    }

    protected override void Dispose (bool disposing)
    {
        if (this.f != null)
        {
            f.Dispose ();
            f = null;
        }
    }
}
