#define NET_1
// #define NET_2

using System;
using System.Drawing;
using System.Windows.Forms;

public enum BezierMode { Create, Edit, Delete };

public class BezierDelegate : Control
{
    private BezierModel model;

    private BezierPoints controlPoints;
    private BezierPoint[] curvePoints;
    private BezierPoint[][] constructionPoints;
    
    private BezierMode mode;

    private Font f;
    private Pen p;

    public BezierDelegate (BezierModel model)
    {
        this.model = model;

        // setup panel
        this.BackColor = Color.White;
        this.mode = BezierMode.Create;
        this.f = new Font ("SansSerif", 8, FontStyle.Regular);
        this.p = new Pen (Color.Black, 2);
        this.controlPoints = null;
        this.curvePoints = null;
        this.constructionPoints = null;

        // connect user interface delegate with model
        this.model.ControlPoints +=
            new ControlPointsEventHandler (this.ControlPointsChanged);
        this.model.CurvePoints +=
            new CurvePointsEventHandler (this.CurvePointsChanged);
        this.model.ConstructionPoints +=
            new ConstructionPointsEventHandler (
            this.ConstructionPointsChanged
        );

        // enable double-buffering
#if NET_1
        this.SetStyle (
            ControlStyles.DoubleBuffer | 
            ControlStyles.UserPaint | 
            ControlStyles.AllPaintingInWmPaint,
            true
        );
#endif

#if NET_2
        this.DoubleBuffered = true;  // .NET Framework Version 2.0
#endif
    }

    public BezierMode Mode
    {
        get
        {
            return this.mode;
        }

        set
        {
            this.mode = value;
            this.Invalidate ();
        }
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        base.OnPaint (e);

        this.DrawControlPoints (e.Graphics);
        this.DrawBezierCurve (e.Graphics);
        this.DrawConstructionPoints (e.Graphics);
    }

    protected override void OnMouseDown (MouseEventArgs e)
    {
        base.OnMouseDown (e);

        if (this.mode == BezierMode.Create)
        {
            this.model.AddPoint (e.X, e.Y);
        }
        else if (this.mode == BezierMode.Edit)
        {
            this.model.MoveBegin (e.X, e.Y);
        }
        else if (this.mode == BezierMode.Delete)
        {
            this.model.RemovePoint (e.X, e.Y);
        }
    }

    protected override void OnMouseUp (MouseEventArgs e)
    {
        base.OnMouseUp (e);

        if (this.mode == BezierMode.Edit) 
        {
            this.model.MoveEnd (e.X, e.Y);
        }
    }

    protected override void OnMouseMove (MouseEventArgs e)
    {
        base.OnMouseMove (e);

        if (e.Button == MouseButtons.None)
        {
            // mouse pointer is moved, no button is pressed
        }
        else if (e.Button == MouseButtons.Left)
        {
            // mouse pointer is moved, left button is pressed
            if (this.mode == BezierMode.Edit)
            {
                this.model.MoveNext (e.X, e.Y);
            }
        }
    }

    private void ControlPointsChanged (BezierPoints controlPoints)
    {
        this.controlPoints = controlPoints;
        this.Invalidate ();
    }

    private void CurvePointsChanged (BezierPoint[] curvePoints)
    {
        this.curvePoints = curvePoints;
        this.Invalidate ();
    }
    
    private void ConstructionPointsChanged (
        BezierPoint[][] constructionPoints
    )
    {
        this.constructionPoints = constructionPoints;
        this.Invalidate ();
    }
    
    private void DrawControlPoints (Graphics g) 
    {
        if (this.controlPoints == null)
            return;

        int numPoints = this.controlPoints.Count;
        if (numPoints == 0)
            return;

        BezierPoint p0, p1;
        p0 = (BezierPoint) this.controlPoints[0];
        this.DrawControlPoint (g, p0.X, p0.Y, Color.DarkRed, "0");

        for (int i = 1; i < numPoints; i ++) 
        {
            p1 = (BezierPoint) this.controlPoints[i];
            String caption = i.ToString ();
            this.DrawControlPoint (g, p1.X, p1.Y, Color.DarkRed, caption);
            g.DrawLine (Pens.Blue, p0.X, p0.Y, p1.X, p1.Y);
            p0 = p1;
        }
    }

    private void DrawControlPoint (
        Graphics g,
        int x,
        int y,
        Color c,
        String caption) 
    {
        this.p.Color = c;
        g.DrawRectangle (this.p, x - 5, y - 5, 10, 10);
        g.DrawString (caption, f, Brushes.Blue, x + 8, y - 8);
    }

    private void DrawBezierCurve (Graphics g) 
    {
        if (this.curvePoints == null)
            return;

        int numPoints = this.curvePoints.Length;
        if (numPoints < 3)
            return;

        BezierPoint p0 = this.curvePoints[0];
        for (int i = 1; i < this.curvePoints.Length; i ++)
        {
            BezierPoint p1 = this.curvePoints[i];
            g.DrawLine (Pens.Blue, p0.X, p0.Y, p1.X, p1.Y);
            p0 = p1;
        }
    }

    private void DrawConstructionPoints (Graphics g) 
    {
        if (this.constructionPoints == null)
            return;

        int numPoints = this.constructionPoints.Length;
        for (int i = numPoints - 2; i >= 0; i--) 
        {
            BezierPoint[] array = this.constructionPoints[i];
            BezierPoint p0 = array[0];
            for (int j = 1; j <= i; j ++) 
            {
                BezierPoint p1 = array[j];
                g.DrawLine (Pens.Black, p0.X, p0.Y, p1.X, p1.Y);
                p0 = p1;
            }
        }

        // drawing Bezier point itself
        BezierPoint pBezier = this.constructionPoints[0][0];
        String caption = "";
        double constructionPosition = this.model.ConstructionPosition;
        if (constructionPosition != 0.0D && constructionPosition != 1.0D)
            caption = String.Format ("{0:0.00}", constructionPosition);
        this.DrawControlPoint (g, pBezier.X, pBezier.Y, Color.Red, caption);
    }

    protected override void Dispose (bool disposing)
    {
        base.Dispose (disposing);

        if (this.f != null)
        {
            f.Dispose ();
            f = null;

            p.Dispose ();
        }
    }
}

