using System;
using System.Drawing;
using System.Windows.Forms;

public class ClearController : Button
{
    private BezierModel model;

    public ClearController (BezierModel model)
    {
        this.model = model;
        this.Text = "Clear";
        this.Width = 80;
    }

    protected override void OnClick (EventArgs e)
    {
        // enable registered clients to receive 'Click' events
        base.OnClick (e);

        this.model.Clear ();
    }
}

public class BezierModeController : ComboBox
{
    private BezierDelegate ui;

    public BezierModeController (BezierDelegate ui)
    {
        this.ui = ui;

        this.Items.Add (BezierMode.Create);
        this.Items.Add (BezierMode.Edit);
        this.Items.Add (BezierMode.Delete);

        this.SelectedItem = BezierMode.Create;
        this.Width = 80;
    }

    protected override void OnSelectedIndexChanged (EventArgs e)
    {
        // enable registered clients to receive 'SelectedItemChanged' events
        base.OnSelectedIndexChanged (e);

        BezierMode mode = (BezierMode) this.SelectedItem;
        this.ui.Mode = mode;
    }
}

public class ResolutionController : Panel
{
    private BezierModel model;

    private Label lResolutionText;
    private Label lResolution;
    private TrackBar tbResolution;

    public ResolutionController (BezierModel model)
    {
        this.model = model;

        // setup container
        this.BorderStyle = BorderStyle.Fixed3D;
        this.ClientSize = new Size (380, 36);

        // setup two labels
        this.lResolutionText = new Label ();
        this.lResolutionText.TextAlign = ContentAlignment.MiddleLeft;
        this.lResolutionText.Text = "Resolution:";
        this.lResolutionText.Width = 62;
        this.lResolutionText.Location = new Point (10, 6);
        
        this.lResolution = new Label ();
        this.lResolution.TextAlign = ContentAlignment.MiddleCenter;
        this.lResolution.BorderStyle = BorderStyle.Fixed3D;
        this.lResolution.Width = 60;
        this.lResolution.Location = new Point (72, 6);

        // setup resolution track bar
        this.tbResolution = new TrackBar ();
        this.tbResolution.Minimum = 1;
        this.tbResolution.Maximum = 100;
        this.tbResolution.LargeChange = 1;
        this.tbResolution.SmallChange = 1;
        this.tbResolution.TickFrequency = 5;
        this.tbResolution.TickStyle = TickStyle.BottomRight;
        this.tbResolution.ValueChanged +=
            new EventHandler (this.TrackBar_ValueChanged);
        this.tbResolution.Value = 50;
        this.tbResolution.Width = 240;
        this.tbResolution.Location = new Point (132, 2);

        // add controls to container
        this.Controls.Add (this.lResolutionText);
        this.Controls.Add (this.lResolution);
        this.Controls.Add (this.tbResolution);
    }

    private void TrackBar_ValueChanged (Object sender, EventArgs e)
    {
        this.lResolution.Text = this.tbResolution.Value.ToString ();

        // inform model
        this.model.Resolution = this.tbResolution.Value;
    }
}

public class ConstructionPositionController : Panel
{
    private BezierModel model;

    private Label lConstructionText;
    private Label lConstruction;
    private TrackBar tbConstruction;

    public ConstructionPositionController (BezierModel model)
    {
        this.model = model;

        // setup container
        this.BorderStyle = BorderStyle.Fixed3D;
        this.ClientSize = new Size (380, 36);

        // setup two labels
        this.lConstructionText = new Label ();
        this.lConstructionText.TextAlign = ContentAlignment.MiddleLeft;
        this.lConstructionText.Text = "t:";
        this.lConstructionText.Width = 62;
        this.lConstructionText.Location = new Point (10, 6);
        
        this.lConstruction = new Label ();
        this.lConstruction.TextAlign = ContentAlignment.MiddleCenter;
        this.lConstruction.BorderStyle = BorderStyle.Fixed3D;
        this.lConstruction.Width = 60;
        this.lConstruction.Location = new Point (72, 6);

        // setup construction position track bar
        this.tbConstruction = new TrackBar ();
        this.tbConstruction.Minimum = 0;
        this.tbConstruction.Maximum = 100;
        this.tbConstruction.LargeChange = 1;
        this.tbConstruction.SmallChange = 1;
        this.tbConstruction.TickFrequency = 5;
        this.tbConstruction.TickStyle = TickStyle.BottomRight;
        this.tbConstruction.ValueChanged +=
            new EventHandler (this.TrackBar_ValueChanged);
        this.tbConstruction.Value = 50;
        this.tbConstruction.Width = 240;
        this.tbConstruction.Location = new Point (132, 2);

        // add controls to container
        this.Controls.Add (this.lConstructionText);
        this.Controls.Add (this.lConstruction);
        this.Controls.Add (this.tbConstruction);
    }

    private void TrackBar_ValueChanged (Object sender, EventArgs e)
    {
        // retrieve current value of track bar
        double value = (((double) this.tbConstruction.Value) / 100);
        this.lConstruction.Text = String.Format ("{0:0.00}", value);

        // inform model
        this.model.ConstructionPosition = value;
    }
}

public class BezierDisplayController : Panel
{
    private BezierModel model;

    private CheckBox cbControlPoints;
    private CheckBox cbCurvePoints;
    private CheckBox cbConstructionPoints;

    public BezierDisplayController (BezierModel model)
    {
        this.model = model;

        // setup container
        this.BorderStyle = BorderStyle.Fixed3D;
        this.ClientSize = new Size (445, 30);

        // setup control points check box
        this.cbControlPoints = new CheckBox ();
        this.cbControlPoints.Checked = model.EnableControlPoints;
        this.cbControlPoints.Text = "Show Control Points";
        this.cbControlPoints.TextAlign = ContentAlignment.MiddleCenter;
        this.cbControlPoints.Width = 130;
        this.cbControlPoints.Location = new Point (10, 4);

        // setup curve points check box
        this.cbCurvePoints = new CheckBox ();
        this.cbCurvePoints.Checked = model.EnableCurvePoints;
        this.cbCurvePoints.Text = "Show Curve Points";
        this.cbCurvePoints.TextAlign = ContentAlignment.MiddleCenter;
        this.cbCurvePoints.Width = 130;
        this.cbCurvePoints.Location = new Point (150, 4);

        // setup construction points check box
        this.cbConstructionPoints = new CheckBox ();
        this.cbConstructionPoints.Checked = model.EnableConstructionPoints;
        this.cbConstructionPoints.Text = "Show Construction Points";
        this.cbConstructionPoints.TextAlign = ContentAlignment.MiddleCenter;
        this.cbConstructionPoints.Width = 160;
        this.cbConstructionPoints.Location = new Point (290, 4);

        // create and register event sink
        EventHandler handler = new EventHandler (this.OnCheckedChanged);
        this.cbControlPoints.CheckedChanged += handler;
        this.cbCurvePoints.CheckedChanged += handler;
        this.cbConstructionPoints.CheckedChanged += handler;

        // add controls to the container
        this.Controls.Add (this.cbControlPoints);
        this.Controls.Add (this.cbCurvePoints);
        this.Controls.Add (this.cbConstructionPoints);
    }

    public void OnCheckedChanged (Object sender, EventArgs e)
    {
        if (sender == this.cbControlPoints)
        {
            this.model.EnableControlPoints =
                this.cbControlPoints.Checked;
        }
        else if (sender == this.cbCurvePoints)
        {
            this.model.EnableCurvePoints =
                this.cbCurvePoints.Checked;
        }
        else if (sender == this.cbConstructionPoints)
        {
            this.model.EnableConstructionPoints =
                this.cbConstructionPoints.Checked;
        }
    }
}
