using System;
using System.Collections;

public class BezierPoint : ICloneable
{
    private int x, y;

    public BezierPoint (int x, int y) 
    {
        this.x = x;
        this.y = y;
    }

    public int X
    {
        get
        {
            return this.x;
        }
    }

    public int Y
    {
        get
        {
            return this.y;
        }
    }

    public Object Clone ()
    {
        return new BezierPoint (this.x, this.y);
    }
}

public class BezierPoints : ArrayList, ICloneable
{
    public BezierPoints () : this (10)
    {
    }

    public BezierPoints (int capacity) : base (capacity)
    {
    }

    public override Object Clone ()
    {
        // deep copy strategy
        BezierPoints points = new BezierPoints (this.Count);
        IEnumerator ienum = this.GetEnumerator ();
        while (ienum.MoveNext ())
        {
            BezierPoint p = (BezierPoint) ienum.Current;
            points.Add (p.Clone ());
        }

        return points;
    }
}

public delegate void ControlPointsEventHandler (BezierPoints points);
public delegate void CurvePointsEventHandler (BezierPoint[] points);
public delegate void ConstructionPointsEventHandler (BezierPoint[][] points);

public interface BezierModel
{
    // properties
    int Resolution { get; set; }
    double ConstructionPosition { get; set; }

    bool EnableControlPoints { get; set; }
    bool EnableCurvePoints { get; set; }
    bool EnableConstructionPoints { get; set; }

    // methods
    void AddPoint (int x, int y);
    bool RemovePoint (int x, int y);
    void Clear ();

    bool MoveBegin (int x, int y);
    bool MoveNext (int x, int y);
    bool MoveEnd (int x, int y);

    // events
    event ControlPointsEventHandler ControlPoints;
    event CurvePointsEventHandler CurvePoints;
    event ConstructionPointsEventHandler ConstructionPoints;
}

public class BezierModelImpl : BezierModel
{
    private BezierPoints controlPoints;

    private bool showControlPoints;
    private bool showCurvePoints;
    private bool showConstructionPoints;

    private int resolution;
    private double constructionPosition;

    private int dragIndex;

    public event ControlPointsEventHandler ControlPoints;
    public event CurvePointsEventHandler CurvePoints;
    public event ConstructionPointsEventHandler ConstructionPoints;
    
    public BezierModelImpl ()
    {
        this.controlPoints = new BezierPoints ();

        this.showControlPoints = true;
        this.showCurvePoints = true;
        this.showConstructionPoints = false;

        this.resolution = 10;
        this.constructionPosition = 0.5;

        this.dragIndex = -1;
    }

    // properties
    public bool EnableControlPoints
    {
        get
        {
            return this.showControlPoints;
        }

        set
        {
            this.showControlPoints = value;
            this.OnStateChange ();
        }
    }

    public bool EnableCurvePoints
    {
        get
        {
            return this.showCurvePoints;
        }

        set
        {
            this.showCurvePoints = value;
            this.OnStateChange ();
        }
    }

    public bool EnableConstructionPoints
    {
        get
        {
            return this.showConstructionPoints;
        }

        set
        {
            this.showConstructionPoints = value;
            this.OnStateChange ();
        }
    }

    public int Resolution
    {
        get
        {
            return this.resolution;
        }

        set
        {
            if (value >= 1 && value <= 100)
            {
                this.resolution = value;
                this.OnStateChange_CurvePoints ();
            }
        }
    }

    public double ConstructionPosition
    {
        get
        {
            return this.constructionPosition;
        }

        set
        {
            if (value >= 0.0 && value <= 1.0)
            {
                this.constructionPosition = value;
                this.OnStateChange_ConstructionPoints ();
            }
        }
    }

    // methods
    public void AddPoint (int x, int y)
    {
        // add new control point
        BezierPoint p = new BezierPoint (x, y);
        this.controlPoints.Add (p);

        this.OnStateChange ();
    }

    public bool MoveBegin (int x, int y)
    {
        BezierPoint p = new BezierPoint (x, y);
        this.dragIndex = this.GetNearestPointIndex (p);
        if (this.dragIndex == -1) 
            return false;

        // move control point
        this.controlPoints[this.dragIndex] = p;

        this.OnStateChange ();
        return true;
    }

    public bool MoveNext (int x, int y)
    {
        if (this.dragIndex == -1)
            return false;

        // update control point
        BezierPoint p = new BezierPoint (x, y);
        this.controlPoints[this.dragIndex] = p;

        this.OnStateChange ();
        return true;
    }

    public bool MoveEnd (int x, int y)
    {
        if (this.dragIndex == -1)
            return false;
        
        // update control point
        BezierPoint p = new BezierPoint (x, y);
        this.controlPoints[this.dragIndex] = p;

        // finish move operation
        this.dragIndex = -1;

        this.OnStateChange ();
        return true;
    }

    public bool RemovePoint (int x, int y)
    {
        // search point
        BezierPoint p = new BezierPoint (x, y);
        int index = this.GetNearestPointIndex (p);
        if (index == -1) 
            return false;

        // delete single point
        this.controlPoints.RemoveAt (index);

        this.OnStateChange ();
        return true;
    }

    public void Clear ()
    {
        // delete all control points
        this.controlPoints.Clear ();

        this.OnStateChange ();
    }

    protected virtual void OnStateChange_ControlPoints ()
    {
        if (this.ControlPoints != null)
        {
            if (this.showControlPoints && this.controlPoints.Count > 0)
            {
                this.ControlPoints (
                    (BezierPoints) this.controlPoints.Clone ()
                );
            }
            else
            {
                this.ControlPoints (null);
            }
        }
    }

    protected virtual void OnStateChange_CurvePoints ()
    {
        if (this.CurvePoints != null)
        {
            if (this.showCurvePoints)
            {
                BezierPoint[] curvePoints = this.ComputeCurvePoints ();
                this.CurvePoints (curvePoints);
            }
            else
            {
                this.CurvePoints (null);
            }
        }
    }

    protected virtual void OnStateChange_ConstructionPoints ()
    {
        if (this.ConstructionPoints != null)
        {
            if (this.showConstructionPoints)
            {
                BezierPoint[][] constructionPoints =
                    this.ComputeConstructionPoints ();
                this.ConstructionPoints (constructionPoints);
            }
            else
            {
                this.ConstructionPoints (null);
            }
        }
    }

    protected virtual void OnStateChange ()
    {
        this.OnStateChange_ControlPoints ();
        this.OnStateChange_CurvePoints ();
        this.OnStateChange_ConstructionPoints ();
    }

    private BezierPoint[] ComputeCurvePoints ()
    {
        if (this.controlPoints == null)
            return null;

        int numPoints = this.controlPoints.Count;
        if (numPoints < 3)
            return null;

        BezierPoint[] points = new BezierPoint[this.resolution + 1];
        BezierPoint p0 = (BezierPoint) this.controlPoints[0];
        points[0] = p0;
        for (int i = 1; i <= this.resolution; i ++) 
        {
            double t = ((double) i) / ((double) this.resolution);
            BezierPoint p1 = this.ComputeSingleCurvePoint (t);
            points[i] = p1;
            p0 = p1;
        }
        return points;
    }

    // helper function to compute a curve point at value t
    private BezierPoint ComputeSingleCurvePoint (double t) 
    {
        int numPoints = this.controlPoints.Count;
        BezierPoint[] array = new BezierPoint[numPoints];
        this.controlPoints.CopyTo (array);

        for (int n = array.Length; n > 0; n --) 
            for (int i = 1; i < n; i ++) 
                array[i-1] = BezierModelImpl.Interpolate (
                    array[i-1], array[i], t
                );

        return array[0];
    }

    // helper function to compute all the construction points
    // needed to compute a single bezier point at value t
    private BezierPoint[][] ComputeConstructionPoints ()
    {
        if (this.controlPoints == null)
            return null;

        int numPoints = this.controlPoints.Count;
        if (numPoints < 3)
            return null;

        BezierPoint[][] constructionPoints = new BezierPoint[numPoints][];
        BezierPoint[] tmpArray = new BezierPoint[numPoints];
        this.controlPoints.CopyTo (tmpArray);
        constructionPoints[numPoints - 1] = tmpArray;

        for (int n = numPoints - 1; n > 0; n --) 
        {
            tmpArray = new BezierPoint[n];
            for (int i = 1; i <= n; i ++) 
            {
                BezierPoint p0 = constructionPoints[n][i-1];
                BezierPoint p1 = constructionPoints[n][i];
                BezierPoint p = BezierModelImpl.Interpolate (
                    p0, p1, this.constructionPosition
                );
                tmpArray[i-1] = p;
            }
            constructionPoints[n - 1] = tmpArray;
        }
        return constructionPoints;
    }

    // helper function to find the nearest control point
    private int GetNearestPointIndex (BezierPoint p0) 
    {
        int numPoints = this.controlPoints.Count;
        if (numPoints == 0)
            return -1;

        int index = -1;
        double dist = Double.MaxValue;

        for (int i = 0; i < numPoints; i ++) 
        {
            BezierPoint p1 = (BezierPoint) this.controlPoints[i];
            double newDist = BezierModelImpl.Distance (p0, p1);
            if (newDist < dist) 
            {
                dist = newDist;
                index = i;
            }
        }
        return (dist < 10.0) ? index : -1;
    }

    // compute the linear interpolation of two points
    public static BezierPoint Interpolate (
        BezierPoint p0,
        BezierPoint p1,
        double t
    )
    {
        double x = t * p1.X + (1 - t) * p0.X;
        double y = t * p1.Y + (1 - t) * p0.Y;
        BezierPoint p = new BezierPoint (
            (int) Math.Round (x), (int) Math.Round (y)
        );
        return p;
    }

    // compute the distance between two points
    public static double Distance (BezierPoint p0, BezierPoint p1) 
    {
        int dx = p1.X - p0.X;
        int dy = p1.Y - p0.Y;
        int sum = dx * dx + dy * dy;
        return Math.Sqrt ((double) sum);
    }
}
