using System;
using System.Drawing;
using System.Windows.Forms;

class GameOfLifePanel : Panel
{
    private bool[,] cells;
    private int[,] tmp;

    public GameOfLifePanel ()
    {
        // panel
        this.Text = "Game of Life";
        this.BackColor = Color.DarkSlateGray;
        this.BorderStyle = BorderStyle.FixedSingle;

        // cell space
        this.cells = new bool[40, 40];
        this.tmp = new int[40, 40];

        // enable double-buffering
        this.SetStyle (
            ControlStyles.DoubleBuffer | 
            ControlStyles.UserPaint | 
            ControlStyles.AllPaintingInWmPaint,
            true
        );
    }

    protected override Size DefaultSize
    {
        get
        {
            return new Size (401, 401);
        }
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        Graphics g = e.Graphics;

        for (int i = 1; i < 40; i ++) 
            g.DrawLine (Pens.DarkGray, 0, 10 * i - 1, 399, 10 * i - 1);

        for (int i = 1; i < 40; i ++) 
            g.DrawLine (Pens.DarkGray, 10 * i - 1, 0, 10 * i - 1, 399);

        for (int i = 0; i < 40; i ++)
            for (int j = 0; j < 40; j ++)
                if (this.cells[i, j])
                    this.DrawCell (g, i, j);
    }

    protected override void OnMouseUp (MouseEventArgs e)
    {
        // enable registered delegates to receive this event
        base.OnMouseUp (e);

        int x = e.X / 10;
        int y = e.Y / 10;
        this.cells[x, y] = ! this.cells[x, y];  // col, row

        // redraw panel
        this.Invalidate ();
    }

    private void DrawCell (Graphics g, int x, int y)
    {
        // calculate absolute coordinates of rectangle
        int px = 10 * x;
        int py = 10 * y;
        g.FillRectangle (Brushes.Yellow, px, py, 9, 9);
    }

    public bool DrawShape (int[] shape, int shapeWidth, int shapeHeight)
    {
        if (shapeWidth > 40 || shapeHeight > 40)
            return false; // shape doesn't fit on panel

        this.Clear ();

        // center shape
        int xOffset = (40 - shapeWidth) / 2;
        int yOffset = (40 - shapeHeight) / 2;
        for (int i = 0; i < shape.Length; i += 2)
            this.cells[xOffset + shape[i], yOffset + shape[i+1]] = true;
        return true;
    }

    public void Clear ()
    {
        // clear cell space
        for (int i = 0; i < 40; i ++)
            for (int j = 0; j < 40; j ++)
                this.cells[i, j] = false;

        // redraw panel
        this.Invalidate ();
    }
        
    public void Next ()
    {
        // Non-standard strategy used here: We're incrementing all the
        // neighbors each time we find a life rather than counting
        // neighbors for each cell.

        // This is faster because it only processes cells that are
        // alive (the obvious algorithm processes all cells).

        // Rather than do a big hairy if statement in a loop, we
        // handle the special cases separately. The special cases
        // are the corners and the four edges less the corners.
        // Once we're done with the special cases, we can handle the
        // main array in a loop.

        // CEEEEC
        // EMMMME    C = corner special case
        // EMMMME    E = edge special case
        // EMMMME    M = main case
        // EMMMME
        // CEEEEC

        // clear temporary buffer
        for (int i = 0; i < 40; i ++)
        {
            for (int j = 0; j < 40; j ++)
            {
                this.tmp[i, j] = 0;
            }
        }

        // increment neighbors' counts for corner cells
        if (this.cells[0, 0])
        { 
            this.tmp[0, 1] += 1;
            this.tmp[1, 0] += 1;
            this.tmp[1, 1] += 1;
        }
        if (this.cells[0, 40 - 1])
        { 
            this.tmp[0, 40 - 2] += 1;
            this.tmp[1, 40 - 1] += 1;
            this.tmp[1, 40 - 2] += 1;
        }
        if (this.cells[40 - 1, 0])
        { 
            this.tmp[40 - 1, 1] += 1;
            this.tmp[40 - 2, 0] += 1;
            this.tmp[40 - 2, 1] += 1;
        }
        if (this.cells[40 - 1, 40 - 1])
        { 
            this.tmp[40 - 1, 40 -2] += 1;
            this.tmp[40 - 2, 40 - 1] += 1;
            this.tmp[40 - 2, 40 -2] += 1;
        }

        // count neighbors of upper and lower edge cells (no corners)
        for (int i = 1; i < 40 - 1; i ++)
        {
            if (this.cells[i, 0])
            {
                this.tmp[i, 1] ++;
                this.tmp[i - 1, 0] ++;
                this.tmp[i - 1, 1] ++;
                this.tmp[i + 1, 0] ++;
                this.tmp[i + 1, 1] ++;
            }

            if (this.cells[i, 40 - 1])
            {
                this.tmp[i, 40 - 2] ++;
                this.tmp[i - 1, 40 - 1] ++;
                this.tmp[i - 1, 40 - 2] ++;
                this.tmp[i + 1, 40 - 1] ++;
                this.tmp[i + 1, 40 - 2] ++;
            }
        }

        // count neighbors of left and right edge cells (no corners)
        for (int j = 1; j < 40 - 1; j ++)
        {
            if (this.cells[0, j])
            {
                this.tmp[1, j] ++;
                this.tmp[0, j - 1] ++;
                this.tmp[1, j - 1] ++;
                this.tmp[0, j + 1] ++;
                this.tmp[1, j + 1] ++;
            }

            if (this.cells[40 - 1, j])
            {
                this.tmp[40 - 2, j] ++;
                this.tmp[40 - 2, j - 1] ++;
                this.tmp[40 - 1, j - 1] ++;
                this.tmp[40 - 2, j + 1] ++;
                this.tmp[40 - 1, j + 1] ++;
            }
        }

        // count neighbors of off-edge cells
        for (int i = 1; i < 40 - 1; i ++)
        {
            for (int j = 1; j < 40 - 1; j ++)
            {
                if (this.cells[i, j])
                {
                    this.tmp[i - 1, j - 1] ++;
                    this.tmp[i , j - 1] ++;
                    this.tmp[i + 1, j - 1] ++;
                    this.tmp[i - 1, j ] ++;
                    this.tmp[i + 1, j ] ++;
                    this.tmp[i - 1, j + 1] ++;
                    this.tmp[i , j + 1] ++;
                    this.tmp[i + 1, j + 1] ++;
                }
            }
        }

        // finally apply game of life algorithm
        for (int i = 0; i < 40; i ++)
        {
            for (int j = 0; j < 40; j ++)
            {
                switch (this.tmp[i, j])
                {
                    case 2:
                        // no change
                        break;
                    case 3:
                        this.cells[i, j] = true;
                        break;
                    default:
                        this.cells[i, j] = false;
                        break;
                }
            }
        }

        // redraw panel
        this.Invalidate ();
    }
}
