using System;
using System.Drawing;
using System.Windows.Forms;

namespace CellularAutomataControls
{
    public delegate void CodeNumberEventHandler (int number);

    public class RulePanel : Control
    {
        private int codeNumber;
        private Rule[] rules;

        public event CodeNumberEventHandler CodeNumberChanged;

        public RulePanel ()
        {
            this.Size = new System.Drawing.Size (480, 64);

            this.rules = new Rule[8];

            RuleStateEventHandler handler =
                new RuleStateEventHandler (this.RuleStateChanged);

            for (int i = 0, x = 16; i < this.rules.Length; i ++)
            {
                Rule rule = new Rule ();
                rule.RulePosition = this.rules.Length - 1 - i;
                rule.RuleState = false;
                rule.RuleStateChanged += handler;

                rule.Location = new Point (x, 16);
                x += 16 * 3 + 9;                

                this.rules[i] = rule;
                this.Controls.Add (rule);
            }

            this.codeNumber = 0;
        }

        private void RuleStateChanged (int pos, bool state)
        {
            // adjust code number
            if (state)
                this.codeNumber |= (0x01 << pos);
            else
                this.codeNumber &= ~(0x01 << pos);

            this.OnCodeNumberChanged (this.codeNumber);
        }
        
        protected override void OnPaint (PaintEventArgs e)
        {
            // take care of registered delegates
            base.OnPaint (e);

            e.Graphics.DrawRectangle (
                Pens.Black,
                0,  // x-coordinate of upper-left corner
                0,  // y-coordinate of upper-left corner
                this.ClientSize.Width - 1,  // width of rectangle
                this.ClientSize.Height - 1  // height of rectangle
            );
        }

        public bool this [int pos]
        {
            get 
            {
                if (pos < 0 || pos >= this.rules.Length)
                {
                    String s = String.Format
                        ("Wrong rule position: {0}", pos);
                    ArgumentOutOfRangeException e = 
                        new ArgumentOutOfRangeException (s);
                    throw (e);
                }
                
                Rule rule = this.rules[this.rules.Length - 1 - pos];
                return rule.RuleState;                
            }

            set
            {
                if (pos < 0 || pos >= this.rules.Length)
                {
                    String s = String.Format ("Wrong rule position: {0}", value);
                    ArgumentOutOfRangeException e = 
                        new ArgumentOutOfRangeException (s);
                    throw (e);
                }
                
                Rule rule = this.rules[this.rules.Length - 1 - pos];
                if (rule.RuleState != value)
                {
                    rule.RuleState = value;
                }
            }
        }

        public int CodeNumber
        {
            get
            {
                return this.codeNumber;
            }

            set
            {
                if (value < 0 || value > 255)
                {
                    String s = String.Format ("Wrong code number: {0}", value);
                    ArgumentOutOfRangeException e = 
                        new ArgumentOutOfRangeException (s);
                    throw (e);
                }

                if (this.codeNumber != value)
                {
                    this.codeNumber = value;

                    for (int i = 0; i < this.rules.Length; i ++)
                        this[i] = (this.codeNumber & (0x01 << i)) != 0;

                    this.OnCodeNumberChanged (this.codeNumber);
                }
            }
        }

        protected virtual void OnCodeNumberChanged (int codeNumber)
        {
            if (this.CodeNumberChanged != null)
                this.CodeNumberChanged (codeNumber);
        }
    }
}
