using System;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Threading;

public class MandelbrotForm : Form
{
    private ASynchronousMandelbrotExcerpt excerpt;
    private Thread t;

    public MandelbrotForm (int width, int height)
    {
        // setup form
        this.Text = "Mandelbrot - Nonblocking Version";
        this.ResizeRedraw = true;
        this.ClientSize = new Size (width, height);
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        if (this.t != null)
        {
            // need to terminate current painting thread
            this.excerpt.Stop ();
            this.t.Join ();
        }

        this.excerpt = new ASynchronousMandelbrotExcerpt (
            this,
            MandelbrotCoords.XMin,  // upper-left corner (x-coordinate)
            MandelbrotCoords.YMin,  // upper-left corner (y-coordinate)
            MandelbrotCoords.XWidth,  // width of complex number area
            MandelbrotCoords.YHeight  // height of complex number area)
        );
        
        // create separate thread context to invoke 'Draw'
        this.t = new Thread (new ThreadStart (this.excerpt.Draw));
        this.t.Start ();
    }

    protected override void OnClosing (CancelEventArgs e)
    {
        base.OnClosing (e);

        if (this.t != null)
        {
            // need to terminate current painting thread
            this.excerpt.Stop ();
            this.t.Join ();
        }
    }
}

class ASynchronousMandelbrotExcerpt
{
    private double xmin;    // upper-left corner (x-coordinate)
    private double ymin;    // upper-left corner (y-coordinate)
    private double xwidth;  // width of complex number area
    private double yheight; // height of complex number area
    private Form   form;    // current 'Form' object
    private bool   isRunning;

    public ASynchronousMandelbrotExcerpt (
        Form form,
        double xmin,
        double ymin,
        double xwidth,
        double yheight)
    {
        this.form = form;
        this.xmin = xmin;
        this.ymin = ymin;
        this.xwidth = xwidth;
        this.yheight = yheight;
        this.isRunning = true;
    }

    public void Stop ()
    {
        this.isRunning = false;
    }

    public virtual void Draw ()
    {
        // create a graphics object for the current control
        Graphics g = this.form.CreateGraphics ();

        // update width and height of client area
        int width = this.form.ClientSize.Width - 1; 
        int height = this.form.ClientSize.Height - 1;

        // create a default brush object
        SolidBrush b = new SolidBrush (Color.Black);

        // apply Mandelbrot formula
        for (int x = 0; x <= width; x ++)
        {
            for (int y = 0; y <= height; y ++)
            {
                // check end of drawing routine
                if (! this.isRunning)
                    goto done;  // transfer control out of nested scope

                // convert pixel position into complex number
                double cx =
                    this.xmin + this.xwidth * ((double) x / width);
                double cy =
                    this.ymin + this.yheight * ((double) y / height);

                // compute numerical order
                double zx = 0.0;
                double zy = 0.0;
                int count = 0;
                while (true)
                {
                    count ++;

                    // apply Mandelbrot formula
                    double tmp = zx;
                    zx = (zx * zx) - (zy * zy) + cx;
                    zy = (2 * tmp * zy) + cy;

                    // Mandelbrot number found
                    if (( zx * zx) + (zy * zy) > 4)
                        break;

                    // it's not a Mandelbrot number
                    if (count >= MandelbrotColors.MaxIterations)
                        break;
                }

                // modify brush
                b.Color = (count >= MandelbrotColors.MaxIterations) ?
                    Color.Black : MandelbrotColors.HSBColors[count];

                // draw pixel
                g.FillRectangle (b, x, y, 1, 1);
            }
        }

        // release resources
    done:
        b.Dispose ();
        g.Dispose ();
    }
}
