using System;
using System.Threading;

class IntArray
{
    private int[] values;
    private int totalSwaps;

    public IntArray (int length)
    {
        this.values = new int[length];
        for (int i = 0; i < values.Length; i ++)
            this.values[i] = values.Length - i; // reverse order

        this.totalSwaps = 0;
    }

    public void VerifyArray ()
    {
        for (int i = 0; i < this.values.Length - 1; i ++)
            if (this.values[i] >= this.values[i+1])
                Console.WriteLine ("Wrong order: {0},{1}: {2},{3}",
                    i, i+1, this.values[i], this.values[i+1]);
    }

    public void PrintStatistics ()
    {
        Console.WriteLine ("Number of total swaps: {0}", this.totalSwaps);
    }

    public virtual bool CompareSwap (int i, int j)
    {
        if (this.values[i] > this.values[j])
        {
            Swap (i, j);
            return true;
        }
        return false;
    }
   
    public virtual void Sort ()
    {
        int id = Thread.CurrentThread.GetHashCode ();
        Console.WriteLine ("Thread {0}: starting ...", id);

        while (true)
        {
            bool sorted = true;
            for (int i = 0; i < this.values.Length - 1; i ++)
                if (this.CompareSwap (i, i+1))
                    sorted = false;

            if (sorted)
                break;
        }

        Console.WriteLine ("Thread {0}: terminating ...", id);
    }

    private void Swap (int i, int j)
    {
        int tmp = this.values[i];
        this.values[i] = this.values[j];
        this.values[j] = tmp;
        this.totalSwaps ++;
    }
}

class SynchronizedIntArray : IntArray
{
    public SynchronizedIntArray (int length) : base (length) {}

    public override bool CompareSwap (int i, int j)
    {
        Monitor.Enter (this);
        bool b = base.CompareSwap (i, j);
        Monitor.Exit (this);
        return b;
    }
}

class OptimizedSynchronizedIntArray : IntArray
{
    private Object[] objects;

    public OptimizedSynchronizedIntArray (int length) : base (length)
    {
        this.objects = new Object[length];
        for (int i = 0; i < this.objects.Length; i ++)
            this.objects[i] = new Object ();
    }

    public override bool CompareSwap (int i, int j)
    {
        Monitor.Enter (this.objects[i]);
        Monitor.Enter (this.objects[j]);
        bool b = base.CompareSwap (i, j);
        Monitor.Exit (this.objects[j]);
        Monitor.Exit (this.objects[i]);
        return b;
    }
}

class CountingOptimizedSynchronizedIntArray
    : OptimizedSynchronizedIntArray
{
    private LocalDataStoreSlot localSlot;

    public CountingOptimizedSynchronizedIntArray (int length)
        : base (length)
    {
        // allocate an unnamed data slot on all the threads
        this.localSlot = Thread.AllocateDataSlot ();
    }

    public override void Sort ()
    {
        // reset thread specific counter
        Thread.SetData (this.localSlot, 0);

        // use sorting algorithm of base class
        base.Sort ();

        // retrieve thread specific counter of swapped elements
        int local = (int) Thread.GetData (this.localSlot);
        Console.WriteLine ("Thread {0}: swapped {1} numbers.",
            Thread.CurrentThread.GetHashCode (), local);
    }

    public override bool CompareSwap (int i, int j)
    {
        bool swapped = base.CompareSwap (i, j);
        if (swapped)
        {
            // increment thread specific counter
            int local = (int) Thread.GetData (this.localSlot);
            local ++;
            Thread.SetData (this.localSlot, local);
        }
        return swapped;
    }
}


