using System;
using System.Threading;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;

class BubbleSortView : Panel
{
    // array to sort
    private int[] numbers;

    // watermarks used during visualization
    private int low;
    private int high;

    // appearance
    private int barHeight;

    // threading utilities
    private Thread t;
    private bool isActive;
    private int interval;
    private bool monitorEnabled;

    public BubbleSortView ()
    {
        // setup control
        this.BorderStyle = BorderStyle.Fixed3D;

        // initialize properties
        this.barHeight = 6;
        this.interval = 0;
        this.monitorEnabled = true;

        // no thread is currently running
        this.isActive = false;

        // generate array with random numbers
        this.InitializeArray (30);

        // control must be redrawn when it is resized
        this.ResizeRedraw = true;

        // enable double-buffering
        this.SetStyle (
            ControlStyles.UserPaint |
            ControlStyles.AllPaintingInWmPaint |
            ControlStyles.DoubleBuffer,
            true
        );
    }

    public int BarHeight
    {
        get
        {
            return this.barHeight;
        }

        set
        {
            this.barHeight = value;
            this.Invalidate ();
        }
    }

    public int Elements
    {
        get
        {
            return this.numbers.Length;
        }

        set
        {
            this.InitializeArray (value);
            this.Invalidate ();
        }
    }

    public int Interval
    {
        get
        {
            return this.interval;
        }

        set
        {
            this.interval = value;
        }
    }

    public bool MonitorEnabled
    {
        get
        {
            return this.monitorEnabled;
        }

        set
        {
            this.monitorEnabled = value;
        }
    }

    public void Start ()
    {
        if (this.isActive)
            return;

        // start animation
        ThreadStart ts = new ThreadStart (this.BubbleSortAnimation);
        this.t = new Thread (ts);
        this.isActive = true;
        this.t.Start ();
    }

    public void Stop ()
    {
        // terminate thread
        if (this.isActive)
        {
            this.isActive = false;

            // wakeup calculation thread, if necessary
            Monitor.Enter (this);
            Monitor.Pulse (this);
            Monitor.Exit (this);

            // wait for end of calculation thread
            this.t.Join ();
        }    
    }

    protected override void OnPaint (PaintEventArgs e)
    {
        // enable registered clients to receive 'Paint' events
        base.OnPaint (e);

        // draw single *consistent* snapshot of bubble sort algorithm
        Monitor.Enter (this);
        for (int i = 0; i < this.numbers.Length; i ++) 
        {
            Brush b = (i == this.low) ?
                Brushes.Yellow : (i == this.high) ?
                Brushes.Red : Brushes.Black;

            this.DrawBar (e.Graphics, b, i, this.numbers[i]);
        }

        // enable next calculation step of bubble sort algorithm
        Monitor.Pulse (this);
        Monitor.Exit (this);
    }

//    protected override void OnPaint (PaintEventArgs e)
//    {
//        // enable registered clients to receive 'Paint' events
//        base.OnPaint (e);
//
//        this.DrawBar (e.Graphics, bBlack, 0,   40);
//        this.DrawBar (e.Graphics, bBlack, 1,   60);
//        this.DrawBar (e.Graphics, bBlack, 2,   80);
//        this.DrawBar (e.Graphics, bBlack, 6,  100);
//        this.DrawBar (e.Graphics, bBlack, 10, 120);
//        this.DrawBar (e.Graphics, bBlack, 12, 140);
//    }

    // helper methods
    private void BubbleSortAnimation ()
    {
        // sort array
        for (int i = this.numbers.Length; -- i >= 0; ) 
        {
            bool ready = true;

            this.high = i;
            for (int j = 0; j < i; j ++) 
            {
                this.low = j + 1;

                // premature end of thread
                if (! this.isActive)
                    return;
                
                // swap two elements thread-safe
                Monitor.Enter (this);
                if (this.numbers[j] > this.numbers[j+1])
                {
                    int tmp = this.numbers[j];
                    this.numbers[j] = this.numbers[j+1];
                    this.numbers[j+1] = tmp;
                    ready = false;
                }

                this.Invalidate ();       // initiate 'OnPaint' invocation
                if (this.monitorEnabled)
                    Monitor.Wait (this);  // wait for end of 'OnPaint'
                Monitor.Exit (this);

                // sleep a short time
                if (this.interval > 0)
                    Thread.Sleep (this.interval);
            }

            if (ready)
                break; // array is sorted
        }

        this.isActive = false;
    }

    public void Reverse ()
    {
        for (int i = 0; i < this.numbers.Length; i ++)
            this.numbers[this.numbers.Length - 1 - i] = 8 * (i + 1);

        // adjust watermarks
        this.low = 0;
        this.high = this.numbers.Length - 1;

        this.Invalidate ();
    }

    public void Shuffle ()
    {
        this.InitializeArray ();
        this.Invalidate ();
    }

    private void InitializeArray ()
    {
        // random-numbers equally distributed between 1 and max
        for (int i = 0; i < this.numbers.Length; i ++)
            this.numbers[i] = 8 * (i + 1);

        Random r = new Random ();
        for (int i = 0; i < this.numbers.Length; i ++)
        {
            int j = r.Next (i);
            int tmp = this.numbers[i];
            this.numbers[i] = this.numbers[j];
            this.numbers[j] = tmp;
        }
        
        // adjust watermarks
        this.low = 0;
        this.high = this.numbers.Length - 1;
    }

    private void InitializeArray (int max) 
    {
        // allocate new array
        this.numbers = new int[max];
        this.InitializeArray ();
    }

    private void DrawBar (Graphics g, Brush b, int index, int width) 
    {
        int x = 20;
        int y = 20 + index * (this.barHeight + 4);
        int w = width * 2;

        g.FillRectangle (b, x, y, w, this.barHeight);
    }

    // release thread resource
    protected override void Dispose (bool disposing)
    {
        this.Stop ();
    }
}
