using System;
using System.Threading;
using System.Text;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Net.Sockets;

// .NET 2.0: Fixing 'InvalidOperationException' crash
delegate void AppendTextHandler (String s);

class ChatClientForm : Form
{
    // UI utilities
    private TextBox tb1;
    private TextBox tb2;
    private TextBox tb3;

    // networking utilities
    private TcpClient client;
    private StreamReader reader;
    private StreamWriter writer;

    // bug fix for .NET 2.0
    private AppendTextHandler handler;

    // threading utilities
    private Thread t;

    public ChatClientForm ()
    {
        // setup form
        this.Text = "Chat Client";
        this.Size = new Size (415, 255);

        // setup label
        Label l1 = new Label ();
        l1.Text = "Name";
        l1.TextAlign = ContentAlignment.MiddleCenter;
        l1.BorderStyle = BorderStyle.Fixed3D;
        l1.Location = new Point (10, 10);
        l1.Size = new Size (85, 20);
        this.Controls.Add (l1);

        // setup 'name' textbox
        this.tb1 = new TextBox ();
        this.tb1.Multiline = false;
        this.tb1.Text = "";
        this.tb1.TextAlign = HorizontalAlignment.Left;
        this.tb1.Location = new Point (100, 10);
        this.tb1.Size = new Size (95, 20);
        this.Controls.Add (this.tb1);

        // setup 'connect' button
        Button b1 = new Button ();
        b1.Text = "Connect";
        EventHandler handler1 =
            new EventHandler (this.ConnectButton_Click);
        b1.Click += handler1;
        b1.Location = new Point (200, 10);
        b1.Size = new Size (95, 20);
        this.Controls.Add (b1);

        // setup 'disconnect' button
        Button b2 = new Button ();
        b2.Text = "Disconnect";
        EventHandler handler2 =
            new EventHandler (this.DisconnectButton_Click);
        b2.Click += handler2;
        b2.Location = new Point (300, 10);
        b2.Size = new Size (95, 20);
        this.Controls.Add (b2);

        // setup chat space textbox
        this.tb2 = new TextBox ();
        this.tb2.Multiline = true;
        this.tb2.ScrollBars = ScrollBars.Vertical;
        this.tb2.Location = new Point (10, 40);
        this.tb2.Size = new Size (385, 150);
        this.Controls.Add (this.tb2);

        // bug fix for .NET 2.0
        this.handler = new AppendTextHandler (this.tb2.AppendText);

        // setup 'send' button
        Button b3 = new Button ();
        b3.Text = "Send";
        EventHandler handler3 =
            new EventHandler (this.SendButton_Click);
        b3.Click += handler3;
        b3.Size = new Size (80, 20);
        b3.Location = new Point (10, 200);
        this.Controls.Add (b3);

        // setup 'send' textbox
        this.tb3 = new TextBox ();
        this.tb3.Multiline = false;
        this.tb3.Text = "";
        this.tb3.Size = new Size (280, 30);
        this.tb3.Location = new Point (100, 200);
        this.Controls.Add (this.tb3);

        // set input focus to 'name' text box
        tb1.Focus ();
    }

    private void OpenConnection ()
    {
        // define a network endpoint (IP address and port number)
        IPAddress adr = IPAddress.Loopback;
        IPEndPoint ep = new IPEndPoint (adr, 4711);

        // create a TcpClient
        this.client = new TcpClient ();

        // connect client to a remote TCP host
        this.client.Connect (ep);

        // retrieve the NetworkStream used to send and receive data
        NetworkStream stream = this.client.GetStream ();

        // create comfortable string input and output objects
        this.reader = new StreamReader (stream, Encoding.ASCII);
        this.writer = new StreamWriter (stream, Encoding.ASCII);

        // flush data to the underlying stream
        // after every call to StreamWriter.WriteLine
        this.writer.AutoFlush = true;
    }

    private void CloseConnection ()
    {
        if (this.client != null)
        {
            // close everything
            this.reader.Close ();
            this.writer.Close ();
            this.client.Close ();

            // wait for the end of worker thread
            this.t.Join ();

            this.reader = null;
            this.writer = null;
            this.client = null;
        }    
    }

    private void Chatting ()
    {
        while (true)
        {
            try
            {
                // read message from chat server, if any
                String response = this.reader.ReadLine ();

                // copy message into text box

                // .NET 1.1: Works fine - although wrong thread context ;-)
                // if (response != null)
                //    this.tb2.AppendText (response + "\r\n");

                // .NET 2.0: Crashes: InvalidOperationException
                // Cross-thread operation not valid: Control 'this.tb2' accessed from a thread
                // other than the thread it was created on.
                if (this.tb2.InvokeRequired)
                {
                    Object[] args = { response + "\r\n" };
                    this.Invoke (this.handler, args);
                }
            }
            catch (IOException)
            {
                // corresponding socket has been closed
                break;
            }
        }
    }

    private void ConnectButton_Click (Object sender, EventArgs e)
    {
        // check for client name
        if (this.tb1.Text == "")
        {
            MessageBox.Show ("Please enter name of client!", "Error");
            return;
        }

        if (this.client == null)
        {
            // establish connection to chat server
            this.OpenConnection ();

            // create worker thread for non-blocking conversation
            ThreadStart ts = new ThreadStart (this.Chatting);
            this.t = new Thread (ts);
            this.t.Start ();
        }
    }

    private void DisconnectButton_Click (Object sender, EventArgs e)
    {
        this.CloseConnection ();
    }

    private void SendButton_Click (Object sender, EventArgs e)
    {
        // send message to server
        if (this.client != null)
        {
            String msg = String.Format ("{0}: {1}",
                this.tb1.Text, this.tb3.Text);

            try
            {
                this.writer.WriteLine (msg);
            }
            catch (IOException)
            {
                // corresponding socket has been closed
                return;
            }
            finally
            {
                this.tb3.Clear ();
            }
        }
    }

    protected override void OnClosing (CancelEventArgs e)
    {
        // enable registered clients to receive 'Closing' event
        base.OnClosing (e);

        // release resources, if any
        this.CloseConnection ();
    }
}
